#ifndef CONTROL_H
#define CONTROL_H
#include <systemc.h>
#include "defs.h"
#include "if.h"

class Control		:public sc_module
{
public:
	sc_out<bool> sourceRun;
	sc_out<bool> encRun[3];
	sc_out<bool> packRun;
	sc_out<bool> packStop;
	sc_port< fifo_reset_if <unsigned char> > fifo_port[3];
	sc_in_clk clk;

	sc_event ev_start, ev_stop;
	int pixelCount;

	bool running;

	Control(sc_module_name name)	: sc_module(name){
		SC_HAS_PROCESS(Control);
		SC_THREAD(main);
			sensitive << clk.pos();
		SC_THREAD(counter);
			sensitive << clk.pos();

		running = 0;
		cout << this->name() << endl;
	}

	void newFrame(){
		if (running == 1){
			cout << "Error: (Control) running\n";
			sc_stop();
		}
		ev_start.notify();
		running = 1;
		cout << "New frame " << endl;
	}
	void endFrame(){
		ev_stop.notify();
		running = 0;
	}
	void counter(){
		wait(ev_start);
		wait(5*CLK);
		while(pixelCount < PIC_SIZE_V*PIC_SIZE_H){
			pixelCount++;
			wait(CLK);
		}
		ev_stop.notify();
		running = 0;
		pixelCount = 0;
	}


	void main(){
		sourceRun->write(0);
		encRun[0]->write(0);
		encRun[1]->write(0);
		encRun[2]->write(0);
		packRun->write(0);
		packStop->write(0);

		wait(ev_start);
		wait(5 * CLK);
		sourceRun->write(1);
		wait(5 * CLK);
		encRun[0]->write(1);
		encRun[1]->write(1);
		encRun[2]->write(1);
		wait(15*CLK);
		packRun->write(1);

		wait(ev_stop);
		sourceRun->write(0);
		wait(5 * CLK);
		encRun[0]->write(0);
		encRun[1]->write(0);
		encRun[2]->write(0);
		wait(11*CLK);
		packRun->write(0);
		packStop->write(1);

		wait(100*CLK);
		if(!DECODE)
			sc_stop();

		fifo_port[0]->reset();
		fifo_port[1]->reset();
		fifo_port[2]->reset();
	}


};

#endif //CONTROL_H
