#ifndef DEC_H
#define DEC_H
#include <systemc.h>
#include "defs.h"
#include "if.h"
#include "fifo.h"
#include "loco.h"
#include "loco_k.h"
#include "golomb_dec.h"
class Dec: public sc_module {
public:

	//SystemC
	sc_in<bool> run;
	sc_in<bool> valid;
	sc_port<fifo_read_if<sc_lv<DATA_SIZE + 1> > > q_in;
	sc_out<bool> ready;
	sc_port<fifo_write_if<unsigned char> > n_out;
	sc_in_clk clk;
	sc_event ev_start;

	//Variables:
	int count, count128;
	sc_lv<DATA_SIZE + 1> tempIn;
	sc_lv<DEC_DATA_SIZE> tempIn16;
	int type;

	//Registers
	sc_lv_reg<DEC_REG_SIZE> reg;
	cache<CACHE_SIZE> * cache_reg;
	ctxt_cache * ctxt_reg;
	sc_lv_reg<DATA_SIZE> reg128;

	//Signals:
	sc_fifo<int> k_fifo;
	sc_fifo<ctxt_info> c_fifo;
	sc_signal<unsigned char> g_sig;

	//LOCO-I
	Loco * iLoco;
	fifo<unsigned char> * dummy_fifo;
	sc_signal<unsigned char> dummy_sig;
	sc_signal<bool> dummy_sig2;
	sc_signal<bool> loco_unenc;
	sc_signal<bool> loco_run;

	//LOCO_K
	Loco_k * iLoco_k;
	sc_signal<bool> loco_k_run;

	//GOLOMB
	Golomb_dec * iGol;
	sc_signal<bool> gol_run;
	sc_fifo<bool> gol_unenc;

	//Unenc

	sc_signal<bool> unenc_run;

	Dec(sc_module_name name, int c) :
			sc_module(name), type(c) {
		SC_HAS_PROCESS(Dec);
		SC_METHOD(append);
		sensitive << clk.neg();
		SC_METHOD(readFifo);
		sensitive << clk.neg();
		SC_METHOD(start);
		sensitive << clk.pos();
		SC_THREAD(proc);
		sensitive << clk.pos();

		//LOCO-I:
		dummy_fifo = new fifo<unsigned char>("dummy_fifo", 1);
		dummy_fifo->clk(clk);
		dummy_fifo->ready(dummy_sig2);
		cache_reg = new cache<CACHE_SIZE>;
		ctxt_reg = new ctxt_cache;
		char temp[20];
		sprintf(temp, "Loco_dec%d", type);
		iLoco = new Loco(temp, type, ctxt_reg, cache_reg, false);
		iLoco->g_in(g_sig);
		iLoco->n_in(*dummy_fifo);
		iLoco->n_out(n_out);
		iLoco->g_out(dummy_sig);
		iLoco->clk(clk);
		iLoco->run(loco_run);
		iLoco->c_in(c_fifo);
		iLoco->unenc_in(gol_unenc);

		//LOCO_K:
		char temp1[20];
		sprintf(temp1, "Loco_k_dec%d", type);
		iLoco_k = new Loco_k(temp1, type, ctxt_reg, cache_reg);
		iLoco_k->c_out(c_fifo);
		iLoco_k->k_out(k_fifo);
		iLoco_k->clk(clk);
		iLoco_k->run(loco_k_run);

		//GOLOMB:
		iGol = new Golomb_dec("Gol_dec", &reg, &count, type);
		iGol->g_out(g_sig);
		iGol->k_in(k_fifo);
		iGol->clk(clk);
		iGol->run(gol_run);
		iGol->unenc_out(gol_unenc);

		count = 0;
		count128 = 0;
	}

	void start() {
		if (run && (count > DEC_REG_SIZE - DEC_DATA_SIZE - 1)) {
			ev_start.notify();
		}
	}

	void proc() {
		while (true) {
			wait(ev_start);
			loco_k_run.write(true);
			wait(6 * CLK);
			gol_run.write(true);
			wait(3 * CLK);
			loco_run.write(true);
			while (run) {
				wait(CLK);
			}
			loco_k_run.write(false);
			wait(2 * CLK);
			gol_run.write(false);
			wait(3 * CLK);
			loco_run.write(false);
		}
	}

	void append() {
		if (count <= DEC_REG_SIZE - DEC_DATA_SIZE - 1) {
			ready.write(true);
			if (run & (count128 > 0)) {
				reg128.get(&tempIn16, DEC_DATA_SIZE);
				count128--;
				reg.add(&tempIn16, DEC_DATA_SIZE);
				count += DEC_DATA_SIZE;
			}
		} else {
			ready.write(false);
		}
	}

	void readFifo() {
		if (valid && run) {
			if (count128 == 0) {
				tempIn = q_in->read();
				reg128.add(&tempIn, DATA_SIZE);
				count128 = DATA_SIZE / DEC_DATA_SIZE;
			}
		}
	}
};

#endif // DEC_H
