#ifndef   	__ENC_H_
#define   	__ENC_H_
#include <systemc.h>
#include "fifo.h"
#include "defs.h"
#include "if.h"
#include "loco.h"
#include "loco_k.h"
#include "golomb.h"


class Enc 	:public sc_module
{
public:

	//SystemC
	sc_port< fifo_read_if < unsigned char > > n_in;
	sc_port< stat > enc_stat;
	sc_out<sc_bv_var> q_out;
	sc_in_clk clk;
	sc_in<bool> run;

	//Variables:

	int pixelCount;
	int type;
//	FILE *f;
	unsigned char X;

	//Signals:
	sc_fifo<int> k_fifo;
	sc_fifo<ctxt_info> c_fifo;
	sc_signal<unsigned char> g_sig;

	//Registers:
	cache<CACHE_SIZE> * reg;
	ctxt_cache * ctxt_reg;

	//LOCO-I:
	Loco * iLoco;
	fifo<unsigned char> * dummy_fifo;
	sc_signal<unsigned char> dummy_sig;
	sc_signal<bool> dummy_sig2;
	sc_fifo<bool> dummy_fifo2;
	sc_signal<bool> loco_run;

	//LOCO_K:
	Loco_k * iLoco_k;
	sc_signal<bool> loco_k_run;

	//Golomb:
	Golomb * iGol;
	sc_signal<bool> gol_run;


	sc_event ev_loco,ev_stop;

	Enc(sc_module_name name, int type_) : sc_module (name), type(type_)	{
    SC_HAS_PROCESS(Enc);
    SC_METHOD(start);
    	sensitive << clk.pos();
    SC_THREAD(proc);
    	sensitive << clk.pos();

    reg = new cache<CACHE_SIZE>;
    ctxt_reg = new ctxt_cache;

    //LOCO_K:
		char temp2[20];
		sprintf(temp2, "Loco_k_enc%d", type);
		iLoco_k = new Loco_k(temp2,type, ctxt_reg,reg);
		iLoco_k->c_out(c_fifo);
		iLoco_k->k_out(k_fifo);
		iLoco_k->clk(clk);
    iLoco_k->run(loco_k_run);


		//LOCO-I:
		dummy_fifo = new fifo<unsigned char>("dummy_fifo",1);
		dummy_fifo->clk(clk);
		dummy_fifo->ready(dummy_sig2);
    char temp[20];
    sprintf(temp, "Loco_enc%d", type);
    iLoco = new Loco(temp,type, ctxt_reg,reg,true);
    iLoco->g_in(dummy_sig);
    iLoco->n_in(n_in);
    iLoco->c_in(c_fifo);
    iLoco->g_out(g_sig);
    iLoco->n_out(*dummy_fifo);
    iLoco->clk(clk);
    iLoco->run(loco_run);
    iLoco->unenc_in(dummy_fifo2);

    //Golomb:
    char temp1[20];
		sprintf(temp1, "Gol_enc%d", type);
    iGol = new Golomb(temp1,type);
    iGol->N(g_sig);
    iGol->k_in(k_fifo);
    iGol->G(q_out);
    iGol->run(gol_run);
    iGol->clk(clk);

		pixelCount = 0;
//		f = fopen(this->name(), "wb");
    cout << this->name() << endl;
  }

  void start(){
		if(run){
			ev_loco.notify();
		}
	}


  void proc(){
		while(true){
			wait(ev_loco);
			loco_k_run.write(true);
			wait(9*CLK);
			loco_run.write(true);
			wait(3*CLK);
			gol_run.write(true);
			while(run){
				pixelCount++;
				sc_bv_var d = q_out.read();
				int si = d.get_size();
	//			fwrite(&si, sizeof(unsigned char), 1, f);
				wait(CLK);
			}
			loco_k_run.write(false);
			wait(4*CLK);
			loco_run.write(false);
			wait(3*CLK);
			gol_run.write(false);
			wait(CLK);
		}
	}
};

#endif 	    /* !ENC_H_ */
