#ifndef FIFO_H
#define FIFO_H

#include <systemc.h>
#include "defs.h"
#include "if.h"

template < class T >
class fifo : public fifo_read_if<T>,
	     public fifo_write_if<T>,
	     public fifo_reset_if<T>,
	     public sc_channel {
 public:
 	sc_in_clk clk;
	sc_out<bool> ready;
  fifo (sc_module_name name, int size_);
  ~fifo ();
  virtual void write (T);
  virtual T read ();
  void reset ();
  int num_available ();
  void checkReady();




 private:
  T *data;
  int num_elements, first;
  int size, num_read, max_used, average;
  int num_overwrite, num_emptyread;
  sc_time last_time;
  void compute_stats ();

  sc_event ev_full;								// instantiating the event objects
	sc_event ev_empty;
	FILE* f;
	void log();
};


template < class T >
fifo<T>::fifo(sc_module_name name, int size_) : sc_channel(name), size(size_) {
	SC_HAS_PROCESS(fifo<T>);
	SC_METHOD(checkReady);
		sensitive << clk;
	SC_METHOD(log);
		sensitive << clk.pos();


  data = new T[size];
  num_elements = first = 0;
  num_read = max_used = average = 0;
  num_overwrite = num_emptyread = 0;
  last_time = SC_ZERO_TIME;

  char tempchar[30];
  sprintf(tempchar, "%s%s.bin\0", OUTPUTPATH, this->name());
  f = fopen(tempchar, "wb");

  cout << this->name();
		  cout << endl;

//  empty.write(1);
}

template < class T >
void fifo<T>::checkReady(){
//	while(true){
		if(num_elements != 0) ready.write(1);
		else ready.write(0);
//	}
}

template < class T >
fifo<T>::~fifo() {
  delete[] data;
}

template < class T >
void fifo<T>::write (T c) {

  if (num_elements == size){
	  if(DECODE)
		wait(ev_full);							// waiting if full
	  else
		num_elements = 1;
	}

    data[(first + num_elements) % size] = c;
	 ++ num_elements;
		ev_empty.notify();						// notifies that it's not empty
}
template < class T >
T fifo<T>::read () {
	T c;
	last_time = sc_time_stamp();
  compute_stats();

  if (num_elements == 0)
  	c = 0;
  else{
		ev_full.notify();											// notifies that it's not full
	  c = data[first];
		-- num_elements;
	  first = (first + 1) % size;
	}

#ifdef TRACE
  cout << "-" << c;
#endif
  return c;
}

template < class T >
void fifo<T>::reset() {
  num_elements = first = 0;
}

template < class T >
int fifo<T>::num_available() {
  return num_elements;
}

template < class T >
void fifo<T>::compute_stats() {
  average += num_elements;
  if (num_elements > max_used)
    max_used = num_elements;
  ++num_read;
}

template < class T >
void fifo<T>::log(){
	fwrite(&num_elements, sizeof(int), 1, f);
}



#endif // FIFO_H
