#ifndef GOLOMB_H
#define GOLOMB_H
#include <systemc.h>
#include "defs.h"
#include "if.h"

class Gol_MUX{
public:
	bool N, s0, s1;
	Gol_MUX();
	bool get(){
		if(s1){
			out = 0;
		}
		else if(s0){
			out = 1;
		}
		else{
			out = N;
		}
		return out;
	}

private:
	bool out;
};

inline Gol_MUX::Gol_MUX(){}

class Golomb 	:public sc_module{
public:
	sc_port<sc_fifo_in_if<int> > k_in;
	sc_in<unsigned char> N;
	sc_in<bool> run;
	sc_out<sc_bv_var > G;
	sc_in_clk clk;

	signed char N_;
	sc_bv<8> q;
	sc_bv<256> n;
	sc_bv_var g;

	int type;
	int k;

	sc_logic temp_n, neg;

	sc_fifo<sc_bv_var> out_fifo;

	int pixelCount;
	int g_max_used;
	int gol_size;

	Gol_MUX * mux[256];

	Golomb(sc_module_name name, int type_) 	:sc_module(name), type(type_){
		SC_HAS_PROCESS(Golomb);
		SC_METHOD(main);
			sensitive << clk.pos();
		SC_THREAD(delay);
			sensitive << clk.pos();

		for(int j = 0; j < 256; j++){
			mux[j] = new Gol_MUX();
		}
		pixelCount = 0;
		g_max_used = 0;
		gol_size = 0;
	}

	~Golomb(){
		cout << "G max used: " << g_max_used << " Comp: " << type << endl;
		cout << "G total size: " << gol_size << " Comp: " << type << endl;
	}

	void main(){
		if (run){
			g.set_size(0);
			k = k_in->read();
			pixelCount++;
			n = N.read();
			neg = n.bit(7);

			for(int i = 0; i < 255; i++){
				temp_n = n.bit(i);
				if(neg.to_bool()){
					temp_n.b_not();
				}
				mux[i]->N = temp_n.to_bool();
				if(i < 7){
					q.bit(i+1) = temp_n;
				}
				if(i == k-1){
					mux[i]->s1 = 1;
					mux[i]->s0 = 0;
				}
				else if(i > k-1){
					mux[i]->s1 = 0;
					mux[i]->s0 = 1;
				}
				else{
					mux[i]->s1 = 0;
					mux[i]->s0 = 0;
				}
				g.bit(i+1) = mux[i]->get();
			}

			sc_logic kb;
			sc_logic b7;
			b7 = n.bit(7);
			if(k==0) kb = 0;
			else kb = 1;
			g.bit(0) = b7 & kb;
			q.bit(0) = b7;
			q.rrotate(k);
			for(int i = 7; i > 7-k; i--){
				q.bit(i) = 0;
			}
			if(q.to_uint() >= SAMPLE_MAX_LENGTH - 9){
				g.set_size(SAMPLE_MAX_LENGTH);
				g.range(7, 0) = n;
				g.range(8,8) = '0';
				for(int i = 9; i < SAMPLE_MAX_LENGTH; i++){
					g.bit(i) = '1';
				}
			}
			else{
				g.set_size(q.to_uint() + 1 + k);
			}
			g.reverse();
			g.lrotate(g.get_size());
			out_fifo.write(g);
			gol_size += g.get_size();
			if(g.get_size() > g_max_used) g_max_used = g.get_size();

							// DEBUG info
							if((pixelCount >= PIXEL-2 && pixelCount <= PIXEL+2)){
										if(type == PIXEL_TYPE){
											cout << sc_time_stamp() << "\t" << this->name() << "\t\t\t";
											cout << "k: " << k << "\t";
											printf("%2X ", (unsigned char)N);
											cout << "s: " << (int)g.get_size() << "\t";
											cout << g.range(g.get_size()-1,0);



											cout << endl;
										}
										}

		}
	}

	void delay(){
		while(true){
			if(run){
				wait(GOL_ENC_DELAY);
				while(true){
					G.write(out_fifo.read());
					wait(CLK);
				}
			}
			wait(CLK);
		}
	}
};

#endif
