#ifndef GOLOMB_DEC_H
#define GOLOMB_DEC_H
#include <systemc.h>
#include "defs.h"
#include "if.h"

class Golomb_dec :public sc_module{
public:
	sc_port<sc_fifo_in_if<int> > k_in;
	sc_in<bool> run;
	sc_out<unsigned char> g_out;
	sc_port<sc_fifo_out_if<bool> > unenc_out;
	sc_in_clk clk;

	sc_bv<8> r_bv;
	sc_lv<8> q_bv;
	sc_lv<8> temp;
	int k;
	int pixelCount;
	int t_count;
	int type;

	sc_fifo<unsigned char> fifo_out;
	sc_fifo<int> fifo_size;
	sc_fifo<bool> fifo_unenc;

	sc_lv_reg<DEC_REG_SIZE> * reg;
	int * count;
	sc_lv<1> qb,rb;
	int q;

	bool unenc;

	Golomb_dec(sc_module_name name, sc_lv_reg<DEC_REG_SIZE> * reg_,int *count_,int type_)
	:sc_module(name)
	,type(type_)
	,reg(reg_)
	,count(count_)
	{
		SC_HAS_PROCESS(Golomb_dec);
		SC_METHOD(main);
			sensitive << clk.pos();
		SC_THREAD(delay);
			sensitive << clk.pos();


		t_count = 0;
		unenc = 0;

	}

	void decrementCount(int s, bool b){
		if(b) *count -= s;
	}


	void main(){
		if(run){
			pixelCount++;
			k = k_in->read();
			if(!unenc){


				for(int i = 0; i < 8; i++){
					r_bv.bit(i) = '0';
					q_bv.bit(i) = '0';
				}

				//q:
				if(reg->checkUnenc(1)) unenc = true;
				reg->get(&qb,1);
				decrementCount(1,!unenc);


				for(q = 0; (qb == '1'); q++){
					if(reg->checkUnenc(1)) unenc = true;
					reg->get(&qb,1);
					decrementCount(1,!unenc);

				}

				q_bv = q;
				if(q == SAMPLE_MAX_LENGTH - 9){
					k = 8;
				}
				q_bv <<= k;


				//r:
				for(int j = 0; j < k; j++){
					if(reg->checkUnenc(1)) unenc = true;
					reg->get(&rb,1);
					r_bv.range(j,j) = rb;
					decrementCount(1,!unenc);

				}
				r_bv.reverse();
				r_bv.lrotate(k);

				if(q == SAMPLE_MAX_LENGTH - 9){
					temp = r_bv;
				}
				else{
					temp = r_bv |= q_bv;

					if(temp.bit(0) == 1){
						temp >>= 1;
						temp = temp.b_not();
					}
					else{
						temp >>= 1;
					}
				}

				if(!unenc){
					fifo_out.write(temp.to_uint());
					fifo_unenc.write(false);
				}

				else{
					reg->setGetUnenc();
					reg->get(&temp,8);
					decrementCount(8,true);
					fifo_out.write(temp.to_uint());
					fifo_unenc.write(true);
					t_count++;
				}
			}
			else{
				if(t_count < PACK_MIN_SIZE-1){
					reg->get(&temp,8);
					decrementCount(8,true);
					fifo_out.write(temp.to_uint());
					fifo_unenc.write(true);
					t_count++;
				}
				else{
					reg->get(&temp,8);
					decrementCount(13,true);
					fifo_out.write(temp.to_uint());
					fifo_unenc.write(true);

					unenc = false;
					reg->clearUnenc();
					t_count = 0;
				}
			}
										// DEBUG info
										if((pixelCount >= PIXEL-2 && pixelCount <= PIXEL+2)){
													if(type == PIXEL_TYPE){
														cout << sc_time_stamp() << "\t" << this->name() << "\t\t\t";
														cout << "k: " << k << "\t\t";
														printf("%2X ", (unsigned char)temp.to_uint());
														cout << *reg << endl;

														sc_lv<DEC_REG_SIZE+1> testtemp;
														testtemp.range(reg->getIndex,reg->getIndex) = '0';
														testtemp.range(reg->addIndex,reg->addIndex) = '0';
														testtemp.range(reg->unencIndex,reg->unencIndex) = '1';
														cout << "\t\t\t\t\t\t\t\t\t  " << testtemp;


														cout << endl;
													}
										}
		}
	}
	void delay(){
		while(true){
			if(run){
				wait(GOL_DEC_DELAY);
				while(true){
					g_out->write(fifo_out.read());
					unenc_out->write(fifo_unenc.read());
					wait(CLK);
				}
			}
			wait(CLK);
		}
	}
};

#endif //GOLOMB_DEC_H

