#include <systemc.h>
#include "loco.h"
Loco::Loco(sc_module_name name, int type_, ctxt_cache *ct_,
		cache<CACHE_SIZE> *reg_, bool enc) :
		ctxt_reg(ct_), reg(reg_), encode(enc), type(type_), sc_module(name) {
	SC_HAS_PROCESS(Loco);
	SC_THREAD(next_pix);
	sensitive << clk.pos();
	SC_THREAD(delay_out);
	sensitive << clk.pos();
	SC_THREAD(start_mod);
	sensitive << clk.pos();

	new_frame();
	cout << this->name() << endl;

	for (int i = 0; i < 365; i++) {
		ctxt_reg->A[i] = 0;
		ctxt_reg->B[i] = 0;
		ctxt_reg->C[i] = 0;
		ctxt_reg->N[i] = 0;
	}

	unenc = 0;
	start_m = false;

//	file = fopen("../testbilder/SC/q_out", "wb");

}

void Loco::new_frame() {

	for (int i = 0; i < PIC_SIZE_H + 2; i++) {
		unsigned char n = 0;
		reg->push(&n);
	}

}

void Loco::start_mod() {
	while (true) {
		if (run) {
			wait(7 * CLK);
			start_m = true;
		}
		wait(CLK);
	}
}

void Loco::next_pix() {
	while (true) {
		if (run) {

			pixelCount++;
			c_in->read(c_);
			if (encode) {
				X = n_in->read();
			} else {
				X = g_in->read();
				unenc_in->read(unenc);
			}

			reg->push(&X);
			reg->get_a(&A);
			reg->get_b(&B);
			reg->get_c(&C);
			reg->get_d(&D);
			reg->get_e(&E);
			reg->get_f(&F);
			reg->get_g(&G);
			reg->get_h(&H);
			reg->get_i(&I);
			reg->get_x(&X);

			predict();
			adj_e();

			if (unenc) {
				fifo_out.write(X);
			} else {
				fifo_out.write(e_);

			}
			fifo_mod.write(e);
			fifo_c.write(c_);
			fifo_ctxt_N.write(ctxt_reg->N[c_.i]);
			fifo_ctxt_B.write(ctxt_reg->B[c_.i]);
			fifo_ctxt_A.write(ctxt_reg->A[c_.i]);
			fifo_ctxt_C.write(ctxt_reg->C[c_.i]);
			wait(1, SC_NS);
			modeling();
			wait(19, SC_NS);

		}
		if (!run) {
			wait(CLK);
		}
	}
}

void Loco::predict() {
	p = A;
}

void Loco::modeling() {

	if (start_m) {
		pixelCountm++;
		temp_e = fifo_mod.read();
		temp_c = fifo_c.read();
		temp_N = fifo_ctxt_N.read();
		temp_A = fifo_ctxt_A.read();
		temp_B = fifo_ctxt_B.read();
		temp_C = fifo_ctxt_C.read();
		if (temp_N >= N0) {
			temp_A = floor((double) temp_A / 2);
			temp_B = floor((double) temp_B / 2);
			temp_N = floor((double) 15);
		}

		temp_B2 = temp_B + temp_e;
		ctxt_reg->B[temp_c.i] = temp_B + temp_e;
		temp_A2 = temp_A + abs(temp_e);
		ctxt_reg->A[temp_c.i] = temp_A + abs(temp_e);
		temp_N2 = temp_N + 1;
		ctxt_reg->N[temp_c.i] = temp_N + 1;

		if (temp_B2 <= -temp_N2) {
			ctxt_reg->C[temp_c.i] = temp_C - 1;
			ctxt_reg->B[temp_c.i] = temp_B2 + temp_N2;
			temp_B3 = temp_B2 + temp_N2;
			if (temp_B3 <= -temp_N2) {
				ctxt_reg->B[temp_c.i] = -temp_N2 + 1;
			}
		} else if (temp_B2 > 0) {
			ctxt_reg->C[temp_c.i] = temp_C + 1;
			ctxt_reg->B[temp_c.i] = temp_B - temp_N;
			temp_B3 = temp_B2 - temp_N2;
			if (temp_B3 > 0) {
				ctxt_reg->B[temp_c.i] = 0;
			}
		}

		if ((pixelCountm >= PIXEL - 2 && pixelCountm <= PIXEL + 2)) {
			if (type == PIXEL_TYPE) {
				cout << sc_time_stamp() << "\t" << this->name() << " mod"
						<< "\t";
				printf("%2X ", (char) temp_e);
				cout << (int) temp_A << "\t";
				cout << (int) temp_B << "\t";
				cout << (int) temp_C << "\t";
				cout << (int) temp_N << "\t";
				cout << endl;
			}
		}
	}
}

void Loco::adj_e() {

	if (encode || unenc) {
		if (c_.sign) {
			e = X - p - ctxt_reg->C[c_.i];
		} else {
			e = -X + p - ctxt_reg->C[c_.i];
		}
		e_ = e;
	}

	else {
		if (c_.sign) {
			e_ = p + X + ctxt_reg->C[c_.i];
		} else {
			e_ = p - X - ctxt_reg->C[c_.i];
		}
		e = X;
		reg->write_x(&e_);
	}
}

void Loco::delay_out() {
	while (true) {
		if (run) {
			wait(LOCO_DELAY);
			while (true) {
				if (encode) {
					e_temp = fifo_out.read();
					g_out->write(e_temp);
				} else {
					n_out->write(fifo_out.read());
				}
				wait(CLK);
			}
		}
		wait(CLK);
	}
}
