#ifndef LOCO_K_H
#define LOCO_K_H
#include <systemc.h>
#include "defs.h"
#include "if.h"
class Loco_k		:public sc_module
{
public:

	sc_in<bool> run;
	sc_port<sc_fifo_out_if<ctxt_info> > c_out;
	sc_port<sc_fifo_out_if<int> > k_out;
	sc_in_clk clk;

	int type;
	cache<CACHE_SIZE> * reg;
	ctxt_cache * ctxt_reg;
	unsigned char A,B,C,D,E,F,G,H,I,X;
	signed int g[3];
	signed int q[3];
	ctxt_info c_;
	int k;

	sc_fifo<ctxt_info> out_fifo;
	sc_fifo<int> k_fifo;
	int pixelCount;

	Loco_k(sc_module_name name, int type_, ctxt_cache *ct_, cache<CACHE_SIZE> *reg_)
		:ctxt_reg(ct_)
		,reg(reg_)
		,type(type_)
		,sc_module(name)
	{
		SC_HAS_PROCESS(Loco_k);
		SC_METHOD(next_pix);
			sensitive << clk.pos();
		SC_THREAD(delay_out);
			sensitive << clk.pos();

		cout << this->name() << endl;

	}
	void next_pix(){
		if(run){
			pixelCount++;

			reg->get_a(&A);
			reg->get_b(&B);
			reg->get_c(&C);
			reg->get_d(&D);
			reg->get_e(&E);
			reg->get_f(&F);
			reg->get_g(&G);
			reg->get_h(&H);
			reg->get_i(&I);
			reg->get_x(&X);

			index();
			comp_k();

			out_fifo.write(c_);
			k_fifo.write(k);

										// DEBUG:
													if((pixelCount >= PIXEL-2 && pixelCount <= PIXEL+2)){
															if(type == PIXEL_TYPE){
																cout << sc_time_stamp() << "\t" << this->name() << "\t";
																cout << "i: " << c_.i << "\t";
																cout << "s: " << c_.sign << "\t";
																cout << "k: " << k << "\t";
										//						cout << (int)ctxt_reg->A[c_.i] << "\t";
										//						cout << (int)ctxt_reg->B[c_.i] << "\t";
										//						cout << (int)ctxt_reg->C[c_.i] << "\t";
										//						cout << (int)ctxt_reg->N[c_.i] << "\t";
																cout << endl;
															}
														}



		}
	}

	void index(){

		g[0] = D-B;
		g[1] = B-C;
		g[2] = I-D;
//		g[2] = 0;

		for(int l = 0; l < 3; l++){
			//if(g[l] <= -21) q[l] = -4;
			if(g[l] <= -7) q[l] = -3;
			else if(g[l] <= -3) q[l] = -2;
			else if(g[l] <= -1) q[l] = -1;
			else if(g[l] <= 0) q[l] = 0;
			else if(g[l] <= 2) q[l] = 1;
			else if(g[l] <= 6) q[l] = 2;
			else if(g[l] <= 20) q[l] = 3;
			else q[l] = 3;
		}

		c_.sign = 1;
		int m = 0;
		if(q[m] == 0) m++;
		if(q[m] == 0) m++;
		if(q[m] < 0){
			c_.sign = 0;
			for(int n = 0; n < 3; n++){
				q[n] = -q[n];
			}
		}
		c_.i = (q[0])*9*9 + (q[1])*9 + (q[2]);
	}

	void comp_k(){
		k = 0;
		while((ctxt_reg->N[c_.i] * pow(2,k)) < ctxt_reg->A[c_.i] && (k < 9))
			++k;
	}

	void delay_out(){
		while(true){
			if(run){
				wait(LOCO_K_DELAY);
				while(true){
					c_out->write(out_fifo.read());
					k_out->write(k_fifo.read());
					wait(CLK);
				}
			}
			wait(CLK);
		}
	}
};


#endif //LOCO_K_H
