#ifndef PACK_H
#define PACK_H
#include <systemc.h>
#include "defs.h"
#include "if.h"

class Pack: public sc_module {
public:

	sc_in<sc_bv_var> sourceP[3];

	sc_in<bool> run;
	sc_in<bool> stop;
	sc_port<fifo_write_if<sc_lv<PACKET_SIZE> > > sinkP;
	sc_port<stat> pack_stat;
	sc_port<sc_fifo_in_if<unsigned char> > t_in[3];
	sc_in_clk clk;

	FILE *f;

	sc_lv_reg<PACK_MEM_SIZE> reg[3];
	sc_bv<PACKET_SIZE - 8> t_reg[3];
	int regCount[3];
	int sampleCount[3];
	int tCount[3];
	int size;
	sc_lv<256> data;
	bool regIndex[3];
	bool add_reg[3];

	sc_logic temp_b;

	bool idle;

	Pack(sc_module_name name) :
			sc_module(name) {
		SC_HAS_PROCESS(Pack);
		SC_METHOD(main);
		sensitive << clk.pos();
		SC_THREAD(_stop)
		sensitive << clk.pos();

		idle = 1;

		for (int i = 0; i < 3; i++) {
			regCount[i] = 0;
			regIndex[i] = 0;
			tCount[i] = 0;
			sampleCount[i] = 0;
			add_reg[i] = true;
		}

		char tempchar[30];
		sprintf(tempchar, "%s%s.bin", OUTPUTPATH, this->name());
		f = fopen(tempchar, "wb");
		cout << this->name() << endl;
	}

	void main() {
		if (run) {
			for (int i = 0; i < 3; i++) {
				if (add_reg[i]) {
					sc_bv_var temp = sourceP[i]->read();
					data = temp;
					size = temp.get_size();
					if (size > SAMPLE_MAX_SIZE)
						size = SAMPLE_MAX_SIZE + 1;
					reg[i].add(&data, size);
					regCount[i] += size;
					tCount[i] = ++sampleCount[i];
					if (sc_time_stamp() >= sc_time(6006400, SC_NS)
							&& sc_time_stamp() <= sc_time(6006900, SC_NS)
							&& i == PIXEL_TYPE) {
						cout << "add pack: " << data.range(size - 1, 0) << endl;
						cout << "tcount: " << tCount[i] << endl;
					}
				} else {
					sc_bv_var temp = sourceP[i]->read();
					tCount[i]++;
				}
		// Bypass original data
		//		unsigned char t_temp = t_in[i]->read();
		//		t_reg[i].rrotate(8);
		//		t_reg[i].range(PACKET_SIZE - 9, PACKET_SIZE - 16) = t_temp;

				if (regCount[i] > DATA_SIZE) {
					writeMem(i);

				}
			}
		}
	}

	void writeMem(int a) {

		sc_lv<PACKET_SIZE> temp = '0';
		if (sc_time_stamp() >= sc_time(6006400, SC_NS)
				&& sc_time_stamp() <= sc_time(6006900, SC_NS)
				&& a == PIXEL_TYPE) {
		}


// Bypass original data if less than PACK_MIN_SIZE
		/*
		 if( sampleCount[a] < (PACK_MIN_SIZE) ){
		 if(sc_time_stamp() >= sc_time(6006400,SC_NS)
		 && sc_time_stamp() <= sc_time(6006900,SC_NS) && a == PIXEL_TYPE){
		 cout << sampleCount[a] << "\t";
		 cout << PACK_MIN_SIZE << endl;
		 }
		 add_reg[a] = false;



		 if(tCount[a] == PACK_MIN_SIZE){
		 sampleCount[a] = 0;
		 regCount[a] = 0;
		 tCount[a] = 0;
		 reg[a].reset();
		 add_reg[a] = true;
		 temp.range(PACKET_SIZE-9,0) = t_reg[a];
		 temp.range(PACKET_SIZE-1,PACKET_SIZE-2) = a;
		 temp.range(PACKET_SIZE-3,PACKET_SIZE-3) = 1;
		 sinkP->write(temp);
		 pack_stat->packer_write();
		 if(sc_time_stamp() >= sc_time(6006400,SC_NS)
		 && sc_time_stamp() <= sc_time(6006900,SC_NS) && a == PIXEL_TYPE){
		 cout << "write pack unenc: " << temp << endl;
		 }
		 }
		 }
		 else{
		 */
//----------------------
		sampleCount[a] = 1;
		regCount[a] -= DATA_SIZE;

		reg[a].get(&temp, DATA_SIZE);
		temp.range(PACKET_SIZE - 1, PACKET_SIZE - 2) = a;
		temp.range(PACKET_SIZE - 3, PACKET_SIZE - 3) = 0;
		if(DECODE) sinkP->write(temp);

		unsigned char pack_hash = 0;
		for(int h = 0; h < DATA_SIZE; h++){
			if(temp.bit(h) == 1){
				pack_hash++;
			}
		}
		fwrite(&pack_hash, sizeof(unsigned char), 1, f);
		pack_stat->packer_write();

		if (sc_time_stamp() >= sc_time(6006600, SC_NS)
				&& sc_time_stamp() <= sc_time(6006900, SC_NS)
				&& a == PIXEL_TYPE) {
			cout << "write pack enc: " << temp << endl;
		}
//		}
	}

	void _stop() {
		while (!stop)
			wait(CLK);

		for (int i = 0; i < 3; i++) {
			writeMem(i);
			regIndex[i] = 0;
			wait(CLK);
		}

	}
};

#endif //PACK_H
