#include <systemc.h>
#include "defs.h"

#include "fifo.h"
#include "time.h"
#include "source.h"
#include "pack.h"
#include "enc.h"
#include "control.h"
#include "control_dec.h"
#include "stat.h"
#include "unpack.h"
#include "dec.h"
#include "sink.h"
#include "cache.h"

// SET IMAGE INPUT/OUTPUT IN DEFS.H

class Top: public sc_module {
public:

	Source * iSource;
	fifo<unsigned char> * iFifo[3];
	fifo<unsigned char> * iFifoOut[3];
	fifo<sc_lv<PACKET_SIZE> > * iFifo128;
	fifo<sc_lv<DATA_SIZE + 1> > * iFifo126[3];
	sc_fifo<unsigned char> * iFifoThrough[3];
	Enc * iEnc[3];
	Pack * iPack;
	Control * iControl;
	ControlDec * iControlDec;
	stat * iStat;
	Unpack * iUnpack;
	Dec * iDec[3];
	Sink * iSink;

	sc_signal<bool> s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s13, s12;
	sc_signal<bool> s11;
	sc_signal<bool> sv1[3], sv3[3], sv4[3], sv5[3], sv6[3], sv7[3], sv8[3];
	sc_signal<sc_bv_var> sv2[3];
	sc_signal<sc_lv<126> > sbv1[3];
	sc_signal<int> sint1;

	Top(sc_module_name name) :
			sc_module(name) {

		SC_HAS_PROCESS(Top);
		SC_THREAD(testbench);

		//Instantiate modules

		iControl = new Control("Control");
		iStat = new stat("stat");
		iSource = new Source("Source");
		iPack = new Pack("Pack");
		iFifo128 = new fifo<sc_lv<PACKET_SIZE> >("Fifo128", FIFO_128_SIZE);

		//decoder
		if (DECODE) {

			iControlDec = new ControlDec("ControlDec");
			iUnpack = new Unpack("unpack");
			iSink = new Sink("Sink");
		}

		char temp1[20];
		char temp2[20];
		char temp3[20];
		char temp4[20];
		char temp5[20];
		char temp6[20];

		for (int i = 0; i < 3; i++) {
			sprintf(temp1, "Fifo8_%d", i);
			iFifo[i] = new fifo<unsigned char>(temp1, FIFO_IN_SIZE);
			sprintf(temp2, "Enc%d", i);
			iEnc[i] = new Enc(temp2, i);
			sprintf(temp6, "FifoThrough_%d", i);
			iFifoThrough[i] = new sc_fifo<unsigned char>();

			if (DECODE) {
				sprintf(temp3, "Dec%d", i);
				iDec[i] = new Dec(temp3, i);
				sprintf(temp4, "FifoOut%d", i);
				iFifoOut[i] = new fifo<unsigned char>(temp4, FIFO_OUT_SIZE);
				sprintf(temp5, "Fifo126%d", i);
				iFifo126[i] = new fifo<sc_lv<DATA_SIZE + 1> >(temp5,
						FIFO_126_SIZE);
			}

		}

		//Port connection
		iControl->clk(C1);
		iControl->sourceRun(s1);
		iControl->packRun(s3);
		iControl->packStop(s4);

		iSource->run(s1);
		iSource->clk(C1);
		iSource->writep_r(*iFifo[0]);
		iSource->writep_g(*iFifo[1]);
		iSource->writep_b(*iFifo[2]);
		iSource->tr_out(*iFifoThrough[0]);
		iSource->tg_out(*iFifoThrough[1]);
		iSource->tb_out(*iFifoThrough[2]);

		iPack->pack_stat(*iStat);
		iPack->run(s3);
		iPack->stop(s4);
		iPack->clk(C1);
		iPack->sinkP(*iFifo128);

		iFifo128->clk(C1);
		iFifo128->ready(s5);

		//Decoder
		if (DECODE) {
			iControlDec->clk(C1);
			iControlDec->sinkRun(s12);
			iControlDec->unpackRun(s13);
			iUnpack->run(s13);
			iUnpack->fifoReady(s5);
			iUnpack->clk(C1);
			iUnpack->sourceP(*iFifo128);

			iSink->readp_r(*(iFifoOut[2]));
			iSink->readp_g(*(iFifoOut[1]));
			iSink->readp_b(*(iFifoOut[0]));
			iSink->dumpP(s10);
			iSink->run(s12);
		}
		for (int i = 0; i < 3; i++) {
			iFifo[i]->ready(sv3[i]);
			iFifo[i]->clk(C1);

			iControl->fifo_port[i](*(iFifo[i]));
			iControl->encRun[i](sv1[i]);

			iEnc[i]->run(sv1[i]);
			iEnc[i]->n_in(*(iFifo[i]));
			iEnc[i]->q_out(sv2[i]);
			iEnc[i]->clk(C1);
			iEnc[i]->enc_stat(*iStat);

			iPack->sourceP[i](sv2[i]);
			iPack->t_in[i](*iFifoThrough[i]);

			//Decoder
			if (DECODE) {
				iControlDec->decRun[i](sv8[i]);
				iDec[i]->run(sv8[i]);
				iDec[i]->clk(C1);
				iDec[i]->ready(sv4[i]);
				iDec[i]->q_in(*(iFifo126[i]));
				iDec[i]->valid(sv6[i]);
				iDec[i]->n_out(*(iFifoOut[i]));

				iUnpack->decReady[i](sv4[i]);
				iUnpack->sinkP[i](*(iFifo126[i]));

				iFifo126[i]->ready(sv6[i]);
				iFifo126[i]->clk(C1);

				iFifoOut[i]->clk(C1);
				iFifoOut[i]->ready(sv7[i]);

				iSink->fifoReady[i](sv7[i]);
			}
		}

		cout << this->name() << endl;
	}

	void testbench() {

		wait(10 * CLK);
		iControl->newFrame();
		wait(3000 * CLK);
		if (DECODE)
			iControlDec->newFrame();
	}
};

int sc_main(int argc, char *argv[]) {
	Top top1("Top1");
	srand((unsigned) time(NULL));

//  sc_start (1000000, SC_NS);
	sc_start();
	delete top1.iStat;
	delete top1.iEnc[0]->iGol;
	delete top1.iEnc[1]->iGol;
	delete top1.iEnc[2]->iGol;
	return 0;
}
