#ifndef SINK_H
#define SINK_H
#include <systemc.h>
#include "defs.h"
#include "if.h"

class Sink: public sc_module {
public:
	sc_port<fifo_read_if<unsigned char> > readp_r, readp_g, readp_b;
	sc_out<bool> dumpP;
	sc_in<bool> fifoReady[3];
	sc_in<bool> run;

	FILE* f;
	FILE* c;
	unsigned char r, g, b;
	unsigned char R, G, B;

	int pixelCount;

	Sink(sc_module_name name) :
			sc_module(name) {
		SC_HAS_PROCESS(Sink);
		SC_THREAD(main);
		SC_THREAD(countLine);

		char tempchar[30];
		sprintf(tempchar, "%s%s", OUTPUTPATH, OUTPUTFILE);
		f = fopen(tempchar, "wb");
		sprintf(tempchar, "%s%s", INPUTPATH, INPUTFILE);
		c = fopen(tempchar, "rb");
		pixelCount = 0;

	}

	void countLine(){
		int lineC = 0;
		while(true){
			if(run){
				cout << INPUTFILE << "- Line number decoded: " << ++lineC << endl;
			}
			wait(1920 * CLK);
		}
	}

	void main() {
		while (true) {
			if (run) {
				pixelCount++;
				R = readp_r->read();
				G = readp_g->read();
				B = readp_b->read();

				if(SPECTRAL_CORR) spectralRecode(R, G, B);

				fwrite(&R, sizeof(unsigned char), 1, f);
				fread(&r, sizeof(unsigned char), 1, c);
				fwrite(&G, sizeof(unsigned char), 1, f);
				fread(&g, sizeof(unsigned char), 1, c);
				fwrite(&B, sizeof(unsigned char), 1, f);
				fread(&b, sizeof(unsigned char), 1, c);
				checkFile();
			}
			wait(CLK);
		}
	}

	void spectralRecode(unsigned char &R, unsigned char &G, unsigned char &B) {
		R = (R + G);
		B = (B + G);
	}

	void checkFile() {

		if (r != R) {
			cout << "ERROR R: " << pixelCount << " ";
			printf("%.2X ", (int) R);
			printf("%.2X ", (int) r);
			cout << endl;
			wait(CLK);
			sc_stop();
		}
		if (g != G) {
			cout << "ERROR G: " << pixelCount << " ";
			printf("%.2X", (int) G);
			printf("%.2X", (int) g);
			cout << endl;
			wait(CLK);
			sc_stop();
		}
		if (b != B) {
			cout << "ERROR B: " << pixelCount << " ";
			printf("%.2X", (int) B);
			printf("%.2X", (int) b);
			cout << endl;
			wait(CLK);
			sc_stop();
		}
	}
};

#endif //SINK_H
