#ifndef   	__STAT_H_
# define   	__STAT_H_

#include <systemc.h>

class stat :  public sc_module, public sc_interface {
 public:


stat (sc_module_name name) {
	encoder_writes = packer_writes = unpacker_reads = decoder_reads = avg_word_length = 0;

	block_size = 8; // block size in bits

	last_write = SC_ZERO_TIME;
	last_read = SC_ZERO_TIME;
}

~stat () {
	cout << endl << "-----------------------" << endl;
	cout << "-- System statistics --" << endl;
	cout << "-- " << INPUTFILE << endl;
	cout << "-----------------------" << endl;
	cout << endl << "Packer writes to mem: " << packer_writes << endl;
	cout << "Compress ratio: 1:" << (double)(PIC_SIZE*24) / (double)(packer_writes*(PACKET_SIZE-HEADER_SIZE)) << endl;
	cout << "Avg bit / sample: " << (double)(packer_writes*PACKET_SIZE-HEADER_SIZE) / (double)(PIC_SIZE*24) * (double)8 << endl;
}

void write (int a) {}
int read () {return 0;}

// Can be called by the encoder when a symbol is written to \ read by the packer
void encoder_write(int word_length) {
	++encoder_writes;
	if (word_length){
		avg_word_length = ((encoder_writes-1)*avg_word_length + word_length)/encoder_writes;
	}
	time_write();
}

// can be called when the packer writes to the fifo (memory)
void packer_write() {
	++packer_writes;
}

void output_packer_writes(){
	cout << endl << "Packer writes to mem: " << packer_writes << endl;
}


// can be called when the decoder reads from the unpacker
void decoder_read() {
	++decoder_reads;
	time_read();
}

// can be called when the unpacker reads from the fifo (memory)
void unpacker_read() {
	++unpacker_reads;
}

void reset() {
	encoder_writes = packer_writes = unpacker_reads = decoder_reads = 0;
	last_write = SC_ZERO_TIME;
	last_read = SC_ZERO_TIME;
}

void time_write() {
	last_write = sc_time_stamp();
}

void time_read() {
	last_read = sc_time_stamp();
}

 private:
	int encoder_writes, packer_writes;
	int decoder_reads, unpacker_reads;
	float avg_word_length;
	int block_size;
	sc_time last_write, last_read;
	void compute_stats();
};

#endif 	    /* !__STAT_H_ */
