library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity ctxt_reg is
	port(
		clk          : in  std_logic;
		rst          : in  std_logic;
		write_0      : in  std_logic;
		write0_addr  : in  std_logic_vector(7 downto 0);

		read_index   : in  std_logic_vector(7 downto 0);
		read_k_index : in  std_logic_vector(7 downto 0);
		write_index  : in  std_logic_vector(7 downto 0);

		A_in         : in  std_logic_vector(A_LEN - 1 downto 0);
		B_in         : in  std_logic_vector(B_LEN - 1 downto 0);
		C_in         : in  std_logic_vector(C_LEN - 1 downto 0);
		N_in         : in  std_logic_vector(N_LEN - 1 downto 0);

		A_out        : out std_logic_vector(A_LEN - 1 downto 0);
		A_k_out      : out std_logic_vector(A_LEN - 1 downto 0);
		B_out        : out std_logic_vector(B_LEN - 1 downto 0);
		C_out        : out std_logic_vector(C_LEN - 1 downto 0);
		N_out        : out std_logic_vector(N_LEN - 1 downto 0);
		N_k_out      : out std_logic_vector(N_LEN - 1 downto 0)
	);
end entity ctxt_reg;

architecture RTL of ctxt_reg is
	type Aarr is array (CTXT_INDEX_RANGE - 1 downto 0) 
	  of std_logic_vector(A_LEN - 1 downto 0);
	type Barr is array (CTXT_INDEX_RANGE - 1 downto 0) 
	  of std_logic_vector(B_LEN - 1 downto 0);
	type Carr is array (CTXT_INDEX_RANGE - 1 downto 0) 
	  of std_logic_vector(C_LEN - 1 downto 0);
	type Narr is array (CTXT_INDEX_RANGE - 1 downto 0) 
	  of std_logic_vector(N_LEN - 1 downto 0);

	signal A          : Aarr;
	signal B          : Barr;
	signal C          : Carr;
	signal N          : Narr;
	signal dataA      : std_logic_vector(A_LEN - 1 downto 0);
	signal dataB      : std_logic_vector(B_LEN - 1 downto 0);
	signal dataC      : std_logic_vector(C_LEN - 1 downto 0);
	signal dataN      : std_logic_vector(N_LEN - 1 downto 0);
	signal write_addr : std_logic_vector(7 downto 0);

begin
	write0 : process(
		A_in, B_in, C_in, N_in, write_0, write0_addr, write_index
	) is
	begin
		if write_0 = '1' then
			dataA      <= (others => '0');
			dataB      <= (others => '0');
			dataC      <= (others => '0');
			dataN      <= (others => '0');
			write_addr <= write0_addr;
		else
			dataA      <= A_in;
			dataB      <= B_in;
			dataC      <= C_in;
			dataN      <= N_in;
			write_addr <= write_index;
		end if;
	end process write0;

	ctxt_mem : process(clk) is
	begin
		if rising_edge(clk) then
			A_out   <= A(conv_integer(unsigned(read_index)));
			A_k_out <= A(conv_integer(unsigned(read_k_index)));
			B_out   <= B(conv_integer(unsigned(read_index)));
			C_out   <= C(conv_integer(unsigned(read_index)));
			N_out   <= N(conv_integer(unsigned(read_index)));
			N_k_out <= N(conv_integer(unsigned(read_k_index)));

			A(conv_integer(unsigned(write_addr))) <= dataA;
			B(conv_integer(unsigned(write_addr))) <= dataB;
			C(conv_integer(unsigned(write_addr))) <= dataC;
			N(conv_integer(unsigned(write_addr))) <= dataN;
		end if;
	end process ctxt_mem;
end architecture RTL;
