library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity dec_modeling is
	port(
		clk        : in  std_logic;
		rst        : in  std_logic;
		data       : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		loco_w_rst : in  std_logic;
		sign       : in  std_logic;
		P          : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		k_int_in   : in  std_logic_vector(3 downto 0);
		k_orig     : in  std_logic_vector(3 downto 0);
		A          : in  std_logic_vector(A_LEN - 1 downto 0);
		B          : in  std_logic_vector(B_LEN - 1 downto 0);
		C          : in  std_logic_vector(C_LEN - 1 downto 0);
		N          : in  std_logic_vector(N_LEN - 1 downto 0);
		A_out      : out std_logic_vector(A_LEN - 1 downto 0);
		B_out      : out std_logic_vector(B_LEN - 1 downto 0);
		C_out      : out std_logic_vector(C_LEN - 1 downto 0);
		N_out      : out std_logic_vector(N_LEN - 1 downto 0);
		Q_out      : out std_logic_vector(SMPL_LEN - 1 downto 0)
	);
end entity dec_modeling;

architecture RTL of dec_modeling is
	signal e       : std_logic_vector(C_LEN downto 0);
	signal A_t1    : std_logic_vector(A_LEN - 1 downto 0);
	signal B_t1    : std_logic_vector(C_LEN downto 0);
	signal C_t1    : std_logic_vector(C_LEN downto 0);
	signal N_t1    : std_logic_vector(C_LEN downto 0);
	signal A_t2    : std_logic_vector(A_LEN - 1 downto 0);
	signal B_t2    : std_logic_vector(C_LEN downto 0);
	signal C_t2    : std_logic_vector(C_LEN downto 0);
	signal N_t2    : std_logic_vector(C_LEN downto 0);
	signal A_t3    : std_logic_vector(A_LEN - 1 downto 0);
	signal B_t3    : std_logic_vector(C_LEN downto 0);
	signal C_t3    : std_logic_vector(C_LEN downto 0);
	signal N_t3    : std_logic_vector(C_LEN downto 0);
	signal e_abs   : std_logic_vector(C_LEN downto 0);
	signal A_t0    : std_logic_vector(A_LEN - 1 downto 0);
	signal B_t0    : std_logic_vector(B_LEN - 1 downto 0);
	signal C_t0    : std_logic_vector(C_LEN - 1 downto 0);
	signal N_t0    : std_logic_vector(N_LEN - 1 downto 0);
	signal P_t0    : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal A_temp  : std_logic_vector(A_LEN - 1 downto 0);
	signal B_temp  : std_logic_vector(C_LEN downto 0);
	signal C_temp  : std_logic_vector(C_LEN downto 0);
	signal N_temp  : std_logic_vector(C_LEN downto 0);
	signal B_t2t1  : std_logic_vector(C_LEN downto 0);
	signal B_t2t2  : std_logic_vector(C_LEN downto 0);
	signal C_t2t1  : std_logic_vector(C_LEN downto 0);
	signal C_t2t2  : std_logic_vector(C_LEN downto 0);
	signal sign_t0 : std_logic;
	signal e_t0    : std_logic_vector(7 downto 0);
	signal data_t1 : std_logic_vector(C_LEN downto 0);
begin
	mapping : process(clk) is
		variable temp2B : std_logic_vector(6 downto 0);
	begin
		if rising_edge(clk) then
			if rst = '1' then
				e_t0 <= (others => '0');
			else
			 -- map error to two sided function:
				if data(0) = '1' then
					e_t0(6 downto 0) <= not data(7 downto 1);
					e_t0(7)          <= '1';
				else
					e_t0(6 downto 0) <= data(7 downto 1);
					e_t0(7)          <= '0';
				end if;
			end if;
		end if;
	end process mapping;

	-- decode input:
	adj_e : process(clk, rst) is
		variable temp_e     : std_logic_vector(C_LEN downto 0);
		variable temp_data1 : std_logic_vector(C_LEN downto 0);
		variable temp_data2 : std_logic_vector(C_LEN downto 0);
	begin
		if rising_edge(clk) then
			if rst = '1' then
				e       <= (others => '0');
				e_abs   <= (others => '0');
				A_t1    <= (others => '0');
				B_t1    <= (others => '0');
				C_t1    <= (others => '0');
				N_t1    <= (others => '0');
				A_t0    <= (others => '0');
				B_t0    <= (others => '0');
				C_t0    <= (others => '0');
				N_t0    <= (others => '0');
				P_t0    <= (others => '0');
				data_t1 <= (others => '0');
				sign_t0 <= '0';
			else
				-- Pipeline signals:
				A_t0    <= A;
				B_t0    <= B;
				C_t0    <= C;
				N_t0    <= N;
				P_t0    <= P;
				sign_t0 <= sign;
				
				-- find absolute value of e:
				temp_e := e_t0(7) & e_t0(7) & e_t0;
				if e_t0(7) = '1' then
					e_abs <= 0 - temp_e;
				else
					e_abs <= temp_e;
				end if;
				e <= temp_e;
				
				-- Find original sample value based on predictor (P) and 
				-- context (C):
				temp_data1 := (P_t0(7) & P_t0(7) & P_t0) 
				  + (C_t0(8) & C_t0) + (e_t0(7) & e_t0(7) & e_t0);
				temp_data2 := (P_t0(7) & P_t0(7) & P_t0) 
				  - (e_t0(7) & e_t0(7) & e_t0) - (C_t0(8) & C_t0);
				
				-- Select right value based on sign value:
				if (sign_t0 = '1') then
					data_t1 <= temp_data1;
				else
					data_t1 <= temp_data2;
				end if;
				
				-- Extend bit lengths and half if N >= 32:
				if N_t0(5) = '0' then
					A_t1 <= A_t0;
					B_t1 <= B_t0(5) & B_t0(5) & B_t0(5) & B_t0(5) & B_t0;
					C_t1 <= C_t0(8) & C_t0;
					N_t1 <= "0000" & N_t0;
				else
					A_t1 <= A_t0(A_LEN - 1) & A_t0(A_LEN - 1 downto 1);
					B_t1 <= B_t0(5) & B_t0(5) & B_t0(5) 
					  & B_t0(5) & B_t0(5) & B_t0(B_LEN - 1 downto 1);
					C_t1 <= C_t0(8) & C_t0;
					N_t1 <= "0000001111";
				end if;
			end if;
		end if;
	end process adj_e;

	modeling : process(clk, rst) is
	begin
		if rising_edge(clk) then
			if loco_w_rst = '1' then
				A_out  <= (others => '0');
				B_out  <= (others => '0');
				C_out  <= (others => '0');
				N_out  <= (others => '0');
				A_t2   <= (others => '0');
				B_t2   <= (others => '0');
				C_t2   <= (others => '0');
				N_t2   <= (others => '0');
				A_t3   <= (others => '0');
				B_t3   <= (others => '0');
				C_t3   <= (others => '0');
				N_t3   <= (others => '0');
				B_t2t1 <= (others => '0');
				B_t2t2 <= (others => '0');
				C_t2t1 <= (others => '0');
				C_t2t2 <= (others => '0');
				A_temp <= (others => '0');
				B_temp <= (others => '0');
				C_temp <= (others => '0');
				N_temp <= (others => '0');
			else
			
			-- Context modeling procedure:
				A_t2   <= A_t1 + e_abs;
				B_t2   <= B_t1 + e;
				C_t2   <= C_t1;
				N_t2   <= N_t1 + 1;
				B_t2t1 <= B_t1 + e + N_t1 + 1;
				B_t2t2 <= B_t1 + e - N_t1 - 1;
				C_t2t1 <= C_t1 - 1;
				C_t2t2 <= C_t1 + 1;
				
				if ((B_t2 <= "0" - N_t2) and B_t2(C_LEN) = '1') then
					A_t3 <= A_t2;
					B_t3 <= B_t2t1;
					C_t3 <= C_t2t1;
					N_t3 <= N_t2;
					if ((B_t3 <= 0 - N_t3) and B_t3(C_LEN) = '1') then
						A_temp <= A_t3;
						B_temp <= 0 - N_t3 + 1;
						C_temp <= C_t3;
						N_temp <= N_t3;
					else
						A_temp <= A_t3;
						B_temp <= B_t3;
						C_temp <= C_t3;
						N_temp <= N_t3;
					end if;
				elsif (B_t2 > 0 and B_t2(C_LEN) = '0') then
					A_t3 <= A_t2;
					B_t3 <= B_t2t2;
					C_t3 <= C_t2t2;
					N_t3 <= N_t2;
					if ((B_t3 <= 0 - N_t3) and B_t3(C_LEN) = '1') then
						A_temp <= A_t3;
						B_temp <= (others => '0');
						C_temp <= C_t3;
						N_temp <= N_t3;
					else
						A_temp <= A_t3;
						B_temp <= B_t3;
						C_temp <= C_t3;
						N_temp <= N_t3;
					end if;
				else
					A_t3   <= A_t2;
					B_t3   <= B_t2;
					C_t3   <= C_t2;
					N_t3   <= N_t2;
					A_temp <= A_t3;
					B_temp <= B_t3;
					C_temp <= C_t3;
					N_temp <= N_t3;
				end if;
				
				-- Output new A,B,C,N:
				A_out  <= A_temp;
				B_out  <= B_temp(B_LEN - 1 downto 0);
				C_out  <= C_temp(C_LEN - 1 downto 0);
				N_out  <= N_temp(N_LEN - 1 downto 0);
			end if;
		end if;
	end process modeling;
	
	-- Output decoded data:
	update_output : process(clk, rst) is
	begin
		if rst = '1' then
			Q_out <= (others => '0');
		elsif rising_edge(clk) then
			Q_out <= data_t1(SMPL_LEN - 1 downto 0);
		end if;
	end process update_output;
end architecture RTL;
