library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity dec_signal_delays is
	port(
		clk        : in  std_logic;
		rst        : in  std_logic;
		index_in   : in  std_logic_vector(7 downto 0);
		sign_in    : in  std_logic;
		k_less_in  : in  std_logic_vector(8 downto 0);
		k_in       : in  std_logic_vector(8 downto 0);
		k_int_in   : in  std_logic_vector(3 downto 0);
		index_out1 : out std_logic_vector(7 downto 0);
		index_out2 : out std_logic_vector(7 downto 0);
		sign_out   : out std_logic;
		k_int_out  : out std_logic_vector(3 downto 0)
	);
end entity dec_signal_delays;

architecture RTL of dec_signal_delays is
	type k_array is array (DEC_K_DELAY downto 0) 
	  of std_logic_vector(8 downto 0);
	type k_less_array is array (DEC_K_DELAY downto 0) 
	  of std_logic_vector(8 downto 0);
	type k_int_array is array (DEC_K_DELAY downto 0) 
	  of std_logic_vector(3 downto 0);
	type index_array is array (DEC_INDEX_DELAY downto 0) 
	  of std_logic_vector(7 downto 0);
	type sign_array is array (DEC_SIGN_DELAY downto 0) 
	  of std_logic;

	signal k      : k_array;
	signal k_less : k_less_array;
	signal k_int  : k_int_array;
	signal index : index_array;
	signal sign  : sign_array;
begin
	propagate_signals : process(clk, rst) is
	begin
		if rising_edge(clk) then

			-- Propagate signals through registers:
			index(DEC_INDEX_DELAY downto 1) 
			  <= index(DEC_INDEX_DELAY - 1 downto 0);
			sign(DEC_SIGN_DELAY downto 1)  
			  <= sign(DEC_SIGN_DELAY - 1 downto 0);
			k(DEC_K_DELAY downto 1)         
			  <= k(DEC_K_DELAY - 1 downto 0);
			k_less(DEC_K_DELAY downto 1)    
			  <= k_less(DEC_K_DELAY - 1 downto 0);
			k_int(DEC_K_DELAY downto 1)     
			  <= k_int(DEC_K_DELAY - 1 downto 0);
			
			-- Input signals
			index(0)   <= index_in;
			k(0)       <= k_in;
			k_less(0)  <= k_less_in;
			k_int(0)   <= k_int_in;
			sign(0)    <= sign_in;
			
			-- Output signals:
			index_out1 <= index(4);
			index_out2 <= index(DEC_INDEX_DELAY);
			sign_out   <= sign(DEC_SIGN_DELAY);
			k_int_out  <= k_int(DEC_K_DELAY);
		end if;
	end process propagate_signals;
end architecture RTL;
