library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity find_end_q is
	port(
		clk       : in  std_logic;
		end_q     : out std_logic_vector(DEC_REG_SIZE - 1 downto 0);
		get_in_in : in  std_logic_vector(DEC_REG_SIZE - 1 downto 0);
		data_in   : in  std_logic_vector(DEC_REG_SIZE - 1 downto 0)
	);
end entity find_end_q;

architecture RTL of find_end_q is
	type end_q_arr is array (DEC_REG_SIZE - 1 downto 0) 
	  of std_logic_vector(7 downto 0);
	signal end_q_n : end_q_arr;
	type data_arr is array (DEC_REG_SIZE - 1 downto 0) 
	  of std_logic_vector(7 downto 0);
	signal data   : data_arr;
	signal get_in : std_logic_vector(DEC_REG_SIZE - 1 downto 0);

begin
	get_in <= get_in_in;

	-- Divide input portitons to the different 'q' counters:
	connect : for n in DEC_REG_SIZE - 8 downto 0 generate
		data(n) <= data_in(n + 7 downto n);
	end generate;
	data(DEC_REG_SIZE - 7) <= data_in(0) 
	  & data_in(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 7);
	data(DEC_REG_SIZE - 6) <= data_in(1 downto 0) 
	  & data_in(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 6);
	data(DEC_REG_SIZE - 5) <= data_in(2 downto 0) 
	  & data_in(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 5);
	data(DEC_REG_SIZE - 4) <= data_in(3 downto 0) 
	  & data_in(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 4);
	data(DEC_REG_SIZE - 3) <= data_in(4 downto 0) 
	  & data_in(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 3);
	data(DEC_REG_SIZE - 2) <= data_in(5 downto 0) 
	  & data_in(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 2);
	data(DEC_REG_SIZE - 1) <= data_in(6 downto 0) 
	  & data_in(DEC_REG_SIZE - 1);
	  
	-- Instantiate n 'q' counters to find end of 'q':
	n_q_reg : for n in DEC_REG_SIZE - 1 downto 0 generate
		q_reg : entity work.count_q port map(
				clk   => clk,
				data  => data(n),
				get   => get_in(n),
				end_q => end_q_n(n)
			);
	end generate;

	-- Select and output the 'q' counter that found the end of 'q':
	output : for n in DEC_REG_SIZE - 1 downto 0 generate
		output_proc : process(end_q_n) is
		begin
			end_q(n) <= (
				end_q_n(n)(0) 
				or end_q_n((n - 1) mod DEC_REG_SIZE)(1) 
				or end_q_n((n - 2) mod DEC_REG_SIZE)(2) 
				or end_q_n((n - 3) mod DEC_REG_SIZE)(3) 
				or end_q_n((n - 4) mod DEC_REG_SIZE)(4) 
				or end_q_n((n - 5) mod DEC_REG_SIZE)(5) 
				or end_q_n((n - 6) mod DEC_REG_SIZE)(6) 
				or end_q_n((n - 7) mod DEC_REG_SIZE)(7)
				);
		end process;
	end generate;
end architecture RTL;
