library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity find_end_r is
	port(
		clk      : in  std_logic;
		end_q    : in  std_logic_vector(DEC_REG_SIZE - 1 downto 0);
		k8       : in  std_logic;
		get_out  : out std_logic_vector(DEC_REG_SIZE - 1 downto 0);
		k_in     : in  std_logic_vector(3 downto 0);
		k_in_bit : in  std_logic_vector(8 downto 0);
		k_out    : out std_logic_vector(3 downto 0)
	);
end entity find_end_r;

architecture RTL of find_end_r is
	type end_r_arr is array (8 downto 0) 
	  of std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal end_r : end_r_arr;
begin

	-- Parallel shift of get index for all 9 'k' values:
	end_r(0) <= end_q(DEC_REG_SIZE - 2 downto 0) 
	  & end_q(DEC_REG_SIZE - 1);
	end_r(1) <= end_q(DEC_REG_SIZE - 3 downto 0) 
	  & end_q(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 2);
	end_r(2) <= end_q(DEC_REG_SIZE - 4 downto 0) 
	  & end_q(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 3);
	end_r(3) <= end_q(DEC_REG_SIZE - 5 downto 0) 
	  & end_q(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 4);
	end_r(4) <= end_q(DEC_REG_SIZE - 6 downto 0) 
	  & end_q(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 5);
	end_r(5) <= end_q(DEC_REG_SIZE - 7 downto 0) 
	  & end_q(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 6);
	end_r(6) <= end_q(DEC_REG_SIZE - 8 downto 0) 
	  & end_q(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 7);
	end_r(7) <= end_q(DEC_REG_SIZE - 9 downto 0) 
	  & end_q(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 8);
	end_r(8) <= end_q(DEC_REG_SIZE - 10 downto 0) 
	  & end_q(DEC_REG_SIZE - 1 downto DEC_REG_SIZE - 9);

	-- Select end_r based on 'k':
	get_out_proc : process(clk, k_in, end_r, k8, k_in_bit) is
	begin
		if k8 = '1' then
			k_out <= k_in;
			for n in 0 to DEC_REG_SIZE - 1 loop
				get_out(n) <= 
				  (end_r(0)(n) and k_in_bit(0)) 
					or (end_r(1)(n) and k_in_bit(1)) 
					or (end_r(2)(n) and k_in_bit(2)) 
					or (end_r(3)(n) and k_in_bit(3)) 
					or (end_r(4)(n) and k_in_bit(4)) 
					or (end_r(5)(n) and k_in_bit(5)) 
					or (end_r(6)(n) and k_in_bit(6)) 
					or (end_r(7)(n) and k_in_bit(7)) 
					or (end_r(8)(n) and k_in_bit(8));
			end loop;
		else
			k_out   <= "1000";
			get_out <= end_r(8);
		end if;
	end process;
end architecture RTL;
