library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity index is
	port(
		clk       : in  std_logic;
		rst       : in  std_logic;
		B_cache   : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		C_cache   : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		D_cache   : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		I_cache   : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		index_out : out std_logic_vector(7 downto 0);
		sign_out  : out std_logic
	);
end entity index;
architecture RTL of index is
	type g_arr is array (0 to 2) 
		of std_logic_vector(SMPL_LEN - 1 downto 0);
	type q_arr is array (0 to 2) of std_logic_vector(2 downto 0);
	signal g          : g_arr;
	signal q          : q_arr;
	signal sign_t1    : std_logic;
	signal test_count : std_logic_vector(7 downto 0);
	signal qn         : q_arr;
begin
	-- compute gradient based on neighbouring pixels:
	gradient : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				g(0) <= (others => '0');
				g(1) <= (others => '0');
				g(2) <= (others => '0');
			else
				g(0) <= D_cache - B_cache;
				g(1) <= B_cache - C_cache;
				g(2) <= I_cache - B_cache;
			end if;
		end if;
	end process gradient;

	-- quantize the gradients:
	quantize : process(clk) is
	begin
		if rising_edge(clk) then
			sign_t1 <= '1';
			for i in 0 to 2 loop
				if (g(i) = "00000000") then				-- g = 0
					q(i)  <= "000";
					qn(i) <= "000";
				elsif (g(i) <= "00000010") then		-- 0 > g >= 2
					q(i)  <= "001";
					qn(i) <= "111";
				elsif (g(i) <= "00000110") then		-- 2 > g >= 6
					q(i)  <= "010";
					qn(i) <= "110";
				elsif (g(i) <= "01111111") then		-- 6 > g >= 127
					q(i)  <= "011";
					qn(i) <= "101";
				elsif (g(i) <= "11101011") then		-- -128 <= g <= -21
					q(i)  <= "101";
					qn(i) <= "011";
					if i = 0 then
						sign_t1 <= '0';
					end if;
				elsif (g(i) <= "11111001") then		-- -21 < g <= -7
					q(i)  <= "101";
					qn(i) <= "011";
					if i = 0 then										-- Set sign bit
						sign_t1 <= '0';
					end if;
				elsif (g(i) <= "11111101") then		-- -7 < g <= -3
					q(i)  <= "110";
					qn(i) <= "010";
					if i = 0 then										-- Set sign bit
						sign_t1 <= '0';
					end if;
				elsif (g(i) <= "11111111") then		-- -3 < g <= -1
					q(i)  <= "111";
					qn(i) <= "001";
					if i = 0 then										-- Set sign bit
						sign_t1 <= '0';
					end if;
				end if;
			end loop;
		end if;
	end process quantize;

	-- Combine quantized regions:
	index : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				index_out  <= (others => '1');
				sign_out   <= '0';
				test_count <= "00000001";
			else
				if (sign_t1 = '0') then
					index_out  <= qn(0)(1 downto 0) & qn(1) & qn(2);
					test_count <= test_count + 1;
				else
					index_out <= q(0)(1 downto 0) & q(1) & q(2);
				end if;
				sign_out <= sign_t1;
			end if;
		end if;
	end process index;
end architecture RTL;
