library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity k8 is
	port(
		clk      : in  std_logic;
		get_in   : in  std_logic_vector(DEC_REG_SIZE - 1 downto 0);
		end_q_in : in  std_logic_vector(DEC_REG_SIZE - 1 downto 0);
		k8       : out std_logic
	);
end entity k8;

architecture RTL of k8 is
	signal k8_arr : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal end_q  : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal get    : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
begin
	get   <= get_in;
	end_q <= end_q_in;
	
	-- Create AND array for all 8 bit distances between get and end_q: 
	find_unenc : process(end_q, get, k8_arr) is
	begin
		for n in 0 to DEC_REG_SIZE - 1 loop
			k8_arr(n) <= get(n) and end_q((n + 7) mod DEC_REG_SIZE);
		end loop;
	end process;
	
	-- If none of the ANDs are '1' output '1', else output '0'
	or_k8 : process(k8_arr, clk) is
	begin
		if k8_arr = 0 then
			k8 <= '1';
		else
			k8 <= '0';
		end if;
	end process or_k8;
end architecture RTL;
