library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity line_cache IS
	port(
		clk        : in  STD_LOGIC;
		shift      : in  STD_LOGIC;
		write_0    : in  std_logic;
		sr_in      : in  STD_LOGIC_VECTOR(SMPL_LEN - 1 downto 0);
		sr_tap_B   : out STD_LOGIC_VECTOR(SMPL_LEN - 1 downto 0);
		sr_tap_C   : out STD_LOGIC_VECTOR(SMPL_LEN - 1 downto 0);
		sr_tap_D   : out STD_LOGIC_VECTOR(SMPL_LEN - 1 downto 0);
		sr_tap_I   : out STD_LOGIC_VECTOR(SMPL_LEN - 1 downto 0);
		sr_tap_B_l : out STD_LOGIC_VECTOR(SMPL_LEN - 1 downto 0)
	);
end line_cache;

architecture arch OF line_cache IS
	--	SUBTYPE sr_width IS STD_LOGIC_VECTOR(SMPL_LEN-1 DOWNTO 0);
	type sr_length is array (CACHE_SIZE - 1 downto 0) 
	  of STD_LOGIC_VECTOR(SMPL_LEN - 1 downto 0);

	signal sr : sr_length;

	signal data : STD_LOGIC_VECTOR(SMPL_LEN - 1 downto 0);

begin
	write0 : process(sr_in, write_0) is
	begin
		if write_0 = '1' then
			data <= (others => '0');
		else
			data <= sr_in;
		end if;
	end process write0;

	process(clk)
	begin
		if (clk'EVENT and clk = '1') then
			if (shift = '1') then
				sr(CACHE_SIZE - 1 downto 1) <= sr(CACHE_SIZE - 2 downto 0);
				sr(0)                       <= data;
			end if;
		end IF;
	end process;
	sr_tap_B   <= sr(CACHE_SIZE - 1920 - CACHE_INDEX_OFFSET - 1);
	sr_tap_C   <= sr(CACHE_SIZE - 1920 - CACHE_INDEX_OFFSET);
	sr_tap_D   <= sr(CACHE_SIZE - 1920 - CACHE_INDEX_OFFSET - 2);
	sr_tap_I   <= sr(CACHE_SIZE - CACHE_INDEX_OFFSET - 1);
	sr_tap_B_l <= sr(CACHE_SIZE - 1920 - 1);
end arch;
