library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity load_data_in is
	port(
		clk     : in  std_logic;
		rst     : in  std_logic;
		get_in  : in  std_logic_vector(DEC_REG_SIZE - 1 downto 0);
		get_out : in  std_logic_vector(DEC_REG_SIZE - 1 downto 0);
		ready   : out std_logic_vector(DATA_IN_REG_NUM - 1 downto 0)
	);
end entity load_data_in;

architecture RTL of load_data_in is
	type bank_arr is array (DATA_IN_REG_NUM - 1 downto 0) of std_logic;
	signal get_out_bank : bank_arr;
	signal get_in_bank  : bank_arr;
	signal get_in_t     : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal get_out_t    : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal ready_t   : std_logic_vector(DATA_IN_REG_NUM - 1 downto 0);
begin
	check_ready : process(clk, get_in, get_out, get_in_bank, 
		get_out_bank, get_in_t, get_out_t, ready_t, rst
	) is
	begin
		for n in 0 to DATA_IN_REG_NUM - 1 loop
			
			-- Reset and update signals:
			if rising_edge(clk) then
				if rst = '1' then
					get_in_t  <= 
					  "000000000000000000000000000000000000000000000001";
					get_out_t <= 
					  "000000000000000000000000000000000000000000000001";
				else
					get_in_t  <= get_in;
					get_out_t <= get_out;
				end if;
			end if;
			
			-- Use banks to determine where get_bit and end_q_bit 
			-- is positioned:
			if get_in_t(
				DATA_IN_REG_SIZE * (n + 1) - 1 downto DATA_IN_REG_SIZE * n
			) = 0 then
				get_in_bank(n) <= '0';
			else
				get_in_bank(n) <= '1';
			end if;
			
			if get_out_t(
				DATA_IN_REG_SIZE * (n + 1) - 1 downto DATA_IN_REG_SIZE * n
			) = 0 then
				get_out_bank(n) <= '0';
			else
				get_out_bank(n) <= '1';
			end if;
      
      -- Check if get_bit and next_bit is in two adjacent registers:
			ready_t(n) <= 
			  get_in_bank(n) and get_out_bank((n + 1) mod DATA_IN_REG_NUM);
			
			-- Output signals:			
			if rst = '1' then
				ready <= "000";
			else
				ready <= ready_t;
			end if;
		end loop;
	end process check_ready;
end architecture RTL;
