library ieee;
use ieee.std_logic_1164.all;
use ieee.STD_LOGIC_ARITH.ALL;
use ieee.std_logic_unsigned.all;
use work.defs.all;

entity enc_signal_delays is
	port(
		clk        : in  std_logic;
		rst        : in  std_logic;
		index_in   : in  std_logic_vector(7 downto 0);
		sign_in    : in  std_logic;
		k_less_in  : in  std_logic_vector(8 downto 0);
		k_in       : in  std_logic_vector(8 downto 0);
		k_int_in   : in  std_logic_vector(3 downto 0);

		index_out1 : out std_logic_vector(7 downto 0);
		index_out2 : out std_logic_vector(7 downto 0);
		sign_out   : out std_logic;
		k_less_out : out std_logic_vector(8 downto 0);
		k_out      : out std_logic_vector(8 downto 0);
		k_int_out  : out std_logic_vector(3 downto 0)
	);
end entity enc_signal_delays;

architecture RTL of enc_signal_delays is
	type k_array is array (ENC_K_DELAY downto 0) 
		of std_logic_vector(8 downto 0);
	type k_less_array is array (ENC_K_DELAY downto 0) 
		of std_logic_vector(8 downto 0);
	type k_int_array is array (ENC_K_DELAY downto 0) 
		of std_logic_vector(3 downto 0);
	type index_array is array (ENC_INDEX_DELAY downto 0) 
		of std_logic_vector(7 downto 0);
	type sign_array is array (ENC_SIGN_DELAY downto 0) 
		of std_logic;

	-- k
	signal k      : k_array;
	signal k_less : k_less_array;
	signal k_int  : k_int_array;

	-- index
	signal index : index_array;
	signal sign  : sign_array;
begin
	propagate_signals : process(clk, rst) is
	begin
		
		if rising_edge(clk) then

			-- Propagate signals through registers:
			index(ENC_INDEX_DELAY downto 1) 
													<= index(ENC_INDEX_DELAY - 1 downto 0);
			sign(ENC_SIGN_DELAY downto 1) 
													<= sign(ENC_SIGN_DELAY - 1 downto 0);
			k(ENC_K_DELAY downto 1) <= k(ENC_K_DELAY - 1 downto 0);
			k_less(ENC_K_DELAY downto 1) 
													<= k_less(ENC_K_DELAY - 1 downto 0);
			k_int(ENC_K_DELAY downto 1)  
													<= k_int(ENC_K_DELAY - 1 downto 0);
			
			-- Input signals:
			index(0)  <= index_in;
			sign(0)   <= sign_in;
			k(0)      <= k_in;
			k_less(0) <= k_less_in;
			k_int(0)  <= k_int_in;
			
			-- Output signals:
			index_out1 <= index(4);
			index_out2 <= index(ENC_INDEX_DELAY);
			sign_out   <= sign(ENC_SIGN_DELAY);
			k_out      <= k(ENC_K_DELAY);
			k_less_out <= k_less(ENC_K_DELAY);
			k_int_out  <= k_int(ENC_K_DELAY);		
		end if;
	end process propagate_signals;
end architecture RTL;


