library ieee;
use ieee.std_logic_1164.all;
use ieee.STD_LOGIC_ARITH.ALL;
use ieee.std_logic_unsigned.all;
use work.defs.all;

entity gol_enc_top is
	port(
		clk        : in  std_logic;
		rst        : in  std_logic;
		k_in       : in  std_logic_vector(8 downto 0);
		k_less_in  : in  std_logic_vector(8 downto 0);
		k_int_in   : in  std_logic_vector(3 downto 0);
		input      : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		B          : in  std_logic_vector(B_LEN - 1 downto 0);
		N          : in  std_logic_vector(N_LEN - 1 downto 0);
		output     : out std_logic_vector(GOL_MAX_LEN - 1 downto 0);
		length_out : out std_logic_vector(3 downto 0)
	);
end entity gol_enc_top;

architecture beh of gol_enc_top is
	signal inReg     : std_logic_vector(GOL_MAX_LEN - 1 downto 0);
	signal k_less    : std_logic_vector(GOL_MAX_LEN - 1 downto 0);
	signal k         : std_logic_vector(GOL_MAX_LEN - 1 downto 0);
	signal k_int_t1  : std_logic_vector(3 downto 0);
	signal reg_t1    : std_logic_vector(GOL_MAX_LEN - 1 downto 0);
	signal input_t1  : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal q_t2      : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal k_less_t1 : std_logic_vector(8 downto 0);
	signal k_t1      : std_logic_vector(8 downto 0);
	signal reg_t2    : std_logic_vector(GOL_MAX_LEN - 1 downto 0);
	signal input_t2  : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal k_int_t2  : std_logic_vector(3 downto 0);
begin
	inReg(GOL_MAX_LEN - 1 downto SMPL_LEN) <= (others => '0');
	inReg(SMPL_LEN - 1 downto 0)           <= input_t1;
	k_less(8 downto 0)                     <= k_less_t1;
	k_less(GOL_MAX_LEN - 1 downto 9)       <= (others => '0');
	k(8 downto 0)                          <= k_t1;
	k(GOL_MAX_LEN - 1 downto 9)            <= (others => '0');
	-- Mapping of twosided function to onesided:
	mapping : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				input_t1 <= (others => '0');
			else
				if input(7) = '1' then
					input_t1(7 downto 1) <= not input(6 downto 0);
					input_t1(0)          <= '1';
				else
					input_t1(7 downto 1) <= input(6 downto 0);
					input_t1(0)          <= '0';
				end if;
			end if;
		end if;
	end process mapping;
	-- Find q:
	shiftreg : entity work.gol_enc_shift port map(
			clk => clk,
			rst => rst,
			k   => k_t1,
			I   => input_t1,
			O   => q_t2
		);
	-- Pipeline registers:
	delay_input : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				input_t2  <= (others => '0');
				k_int_t1  <= (others => '0');
				k_int_t2  <= (others => '0');
				k_less_t1 <= (others => '0');
				k_t1      <= "000000001";
				reg_t2    <= (others => '0');
			else
				input_t2  <= input_t1;
				k_int_t1  <= k_int_in;
				k_int_t2  <= k_int_t1;
				k_less_t1 <= k_less_in;
				k_t1      <= k_in;
				reg_t2    <= reg_t1;
			end if;
		end if;
	end process delay_input;
	
	-- Instantiate n number of muxes to compute golomb code:
	n_mux_modules : for n in GOL_MAX_LEN - 1 downto 0 generate
		mux_module : entity work.gol_enc_mux port map(
				D(0) => k_less(n),
				D(1) => k(n),
				Y    => reg_t1(n),
				S    => inReg(n)
			);
	end generate;

	-- Find length and output data: 
	reduce : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				length_out <= (others => '0');
				output     <= (others => '0');
			else
				if (q_t2 > 6) then -- If q longer than gol max length
					length_out <= "1111";
					output     <= "11111110" & input_t2;
				else
					length_out <= q_t2(3 downto 0) + k_int_t2;
					output     <= reg_t2;
				end if;
			end if;
		end if;
	end process reduce;
end beh;

