library ieee;

use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity dec_loco is
	port(
		clk        : in  std_logic;
		rst        : in  std_logic;
		data       : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		P          : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		Q_out      : out std_logic_vector(SMPL_LEN - 1 downto 0)
	);

end entity dec_loco;

architecture RTL of dec_loco is
	signal e : std_logic_vector(C_LEN downto 0);
	signal e_abs : std_logic_vector(C_LEN downto 0);
	signal P_t0  : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal e_t0    : std_logic_vector(7 downto 0);
	signal data_t1 : std_logic_vector(C_LEN downto 0);

begin
	mapping : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				e_t0 <= (others => '0');
			else
				if data(0) = '1' then
					e_t0(6 downto 0) <= not data(7 downto 1);
					e_t0(7)          <= '1';
				else
					e_t0(6 downto 0) <= data(7 downto 1);
					e_t0(7)          <= '0';
				end if;

			end if;
		end if;
	end process mapping;

	adj_e : process(clk, rst) is
		variable temp_e     : std_logic_vector(C_LEN downto 0);

	begin
		if rising_edge(clk) then
			if rst = '1' then
				e       <= (others => '0');
				e_abs   <= (others => '0');
				P_t0    <= (others => '0');
				data_t1 <= (others => '0');
			else
				P_t0    <= P;

				temp_e := e_t0(7) & e_t0(7) & e_t0;
				if e_t0(7) = '1' then
					e_abs <= 0 - temp_e;
				else
					e_abs <= temp_e;
				end if;
				e <= temp_e;

				data_t1 <= (P_t0(7) & P_t0(7) & P_t0) + (e_t0(7) & e_t0(7) & e_t0);

			end if;
		end if;
	end process adj_e;

	update_output : process(clk, rst) is
	begin
		if rst = '1' then
			Q_out <= (others => '0');
		elsif rising_edge(clk) then
			Q_out <= data_t1(SMPL_LEN - 1 downto 0);
		end if;
	end process update_output;
end architecture RTL;
