library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity dec_reset is
	port(
		clk        : in  std_logic;
		rst        : in  std_logic;
		new_frame  : in  std_logic;
		ready      : out std_logic;
		cache_rst  : out std_logic;
		loco_rst   : out std_logic;
		gol_rst    : out std_logic
	);
end entity dec_reset;

architecture RTL of dec_reset is
	signal counter     : std_logic_vector(12 downto 0);
	signal index_rst0  : std_logic_vector(DEC_INDEX_RST downto 0);
	signal loco_rst0   : std_logic_vector(DEC_LOCO_RST downto 0);
	signal gol_rst0    : std_logic_vector(DEC_GOL_RST downto 0);
begin
	empty_cache : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				counter   <= (others => '0');
			else
				if new_frame = '1' then
					counter   <= counter + 1;
				else
					counter   <= (others => '0');
				end if;
			end if;
		end if;
	end process empty_cache;

	name : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				ready       <= '0';
				loco_rst    <= '1';
				gol_rst     <= '1';
				index_rst0  <= (others => '1');
				loco_rst0   <= (others => '1');
				gol_rst0    <= (others => '1');
			else
				if new_frame = '1' then
					if counter(12) = '1' then
						ready                                <= '1';
						index_rst0(0)                        <= '0';
						loco_rst0(0)                         <= '0';
						gol_rst0(0)                          <= '0';
						index_rst0(DEC_INDEX_RST downto 1)   <= index_rst0(DEC_INDEX_RST - 1 downto 0);
						loco_rst0(DEC_LOCO_RST downto 1)     <= loco_rst0(DEC_LOCO_RST - 1 downto 0);
						gol_rst0(DEC_GOL_RST downto 1)       <= gol_rst0(DEC_GOL_RST - 1 downto 0);
					else
						ready       <= '0';
						index_rst0  <= (others => '1');
						loco_rst0   <= (others => '1');
						gol_rst0    <= (others => '1');
					end if;
				else
					ready                                <= '0';
					index_rst0(0)                        <= '0';
					loco_rst0(0)                         <= '0';
					gol_rst0(0)                          <= '0';
					index_rst0(DEC_INDEX_RST downto 1)   <= index_rst0(DEC_INDEX_RST - 1 downto 0);
					loco_rst0(DEC_LOCO_RST downto 1)     <= loco_rst0(DEC_LOCO_RST - 1 downto 0);
					gol_rst0(DEC_GOL_RST downto 1)       <= gol_rst0(DEC_GOL_RST - 1 downto 0);
				end if;
				loco_rst   <= loco_rst0(DEC_LOCO_RST);
				gol_rst    <= gol_rst0(DEC_GOL_RST);
				cache_rst  <= index_rst0(DEC_INDEX_RST);
			end if;
		end if;
	end process name;
end architecture RTL;
