library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity gol_dec_top is
	port(
		clk       : in  std_logic;
		rst       : in  std_logic;
		-- write data:
		q_out     : out std_logic_vector(2 downto 0);
		r_out     : out std_logic_vector(7 downto 0);
		data_out  : out std_logic_vector(7 downto 0);
		-- get data:
		data_in   : in  std_logic_vector(DATA_IN_REG_SIZE - 1 downto 0);
		ready_out : out std_logic
	);
end entity gol_dec_top;

architecture RTL of gol_dec_top is
	signal get_in      : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal end_q_out   : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal get_out     : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal end_q_in    : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal q           : std_logic_vector(2 downto 0);
	signal r           : std_logic_vector(7 downto 0);
	signal get_in_prev : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal q_temp      : std_logic_vector(2 downto 0);
	signal r_temp1     : std_logic_vector(7 downto 0);
	signal r_temp2     : std_logic_vector(7 downto 0);
	signal ready       : std_logic_vector(DATA_IN_REG_NUM - 1 downto 0);
	signal data        : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal data_r      : std_logic_vector(DEC_REG_SIZE - 1 downto 0);
	signal k_out       : std_logic_vector(3 downto 0);
	signal k_in_t1     : std_logic_vector(3 downto 0);
	signal k_in_t2     : std_logic_vector(3 downto 0);

begin
	name : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				get_in(DEC_REG_SIZE - 1 downto 1)      <= (others => '0');
				get_in(0)                              <= '1';
				get_in_prev(DEC_REG_SIZE - 1 downto 1) <= (others => '0');
				get_in_prev(0)                         <= '1';
				end_q_in                               <= (others => '0');

			else
				get_in      <= get_out;
				end_q_in    <= end_q_out;
				get_in_prev <= get_in;
			end if;
			q_temp  <= q;
			r_temp1 <= r;
			r_temp2 <= r_temp1;
			data_r  <= data;
			k_in_t1 <= k_out;
			k_in_t2 <= k_in_t1;
		end if;
	end process name;

	data_in_reg_module : entity work.data_in_reg port map(
			clk      => clk,
			rst      => rst,
			data_in  => data_in,
			ready_in => ready,
			data_out => data
		);

	count_module : entity work.update_registers port map(
			clk       => clk,
			get_in    => get_in,
			data_in   => data,
			end_q_out => end_q_out,
			get_out   => get_out,
			k_out     => k_out
		);
	load_data_in_module : entity work.load_data_in port map(
			clk     => clk,
			rst     => rst,
			get_in  => get_in,
			get_out => get_out,
			ready   => ready
		);
	ready_out <= ready(0) or ready(1) or ready(2);

	decode_q_module : entity work.decode_q port map(
			clk      => clk,
			rst      => rst,
			end_q_in => end_q_in,
			get_in   => get_in_prev,
			q_out    => q
		);
	decode_r_module : entity work.decode_r port map(
			clk      => clk,
			rst      => rst,
			end_q_in => end_q_in,
			data_in  => data_r,
			data_out => r
		);
	comb_qr_module : entity work.comb_qr port map(
			clk      => clk,
			rst      => rst,
			k        => k_in_t2,
			q        => q,
			r        => r_temp1,
			data_out => data_out
		);
end architecture RTL;
