library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity enc_top is
	port(
		clk       : in  std_logic;
		rst       : in  std_logic;
		data_in   : in  std_logic_vector(7 downto 0);
		new_frame : in  std_logic;
		ready     : out std_logic;
		data_out  : out std_logic_vector(GOL_MAX_LEN - 1 downto 0);
		size_out  : out std_logic_vector(3 downto 0)
	);
end entity enc_top;

architecture RTL of enc_top is
	signal data_in_c : std_logic_vector(7 downto 0);


	--loco
	signal loco_rst : std_logic;
	signal B_l        : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal Q          : std_logic_vector(SMPL_LEN - 1 downto 0);

	--gol
	signal gol_rst  : std_logic;

	--output
	signal data_out_c : std_logic_vector(GOL_MAX_LEN - 1 downto 0);
	signal size_out_c : std_logic_vector(3 downto 0);

	signal cache_rst  : std_logic;
	signal not_rst    : std_logic;
	signal data_in_c1 : std_logic_vector(7 downto 0);
	signal data_in_c2 : std_logic_vector(7 downto 0);
	signal data_in_c3 : std_logic_vector(7 downto 0);
begin
	not_rst <= not rst;

	clocked_in_out : process(clk, rst) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				data_in_c <= (others => '0');
				data_out  <= (others => '0');
				size_out  <= (others => '0');
			else
				data_in_c  <= data_in;
				data_in_c3 <= data_in_c2;
				data_in_c2 <= data_in_c1;
				data_in_c1 <= data_in_c;

				data_out <= data_out_c;
				size_out <= size_out_c;
			end if;
		end if;
	end process clocked_in_out;


	loco_module : entity work.loco port map(
			clk        => clk,
			rst        => loco_rst,
			data       => data_in_c,
			P          => B_l,
			Q_out      => Q
		);
	gol_module : entity work.gol_enc port map(
			clk        => clk,
			rst        => gol_rst,
			input      => Q,
			output     => data_out_c,
			length_out => size_out_c
		);

	cache_module : entity work.line_cache port map(
			clk        => clk,
			shift      => not_rst,
			write_0    => cache_rst,
			sr_in      => data_in_c3,
			sr_tap_B_l => B_l
		);

	reset_module : entity work.reset port map(
			clk        => clk,
			rst        => rst,
			new_frame  => new_frame,
			ready      => ready,
			cache_rst  => cache_rst,
			loco_rst   => loco_rst,
			gol_rst    => gol_rst
		);
end architecture RTL;
