library ieee;
use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity gol_enc is
	port(
		clk        : in  std_logic;
		rst        : in  std_logic;
		input      : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		output     : out std_logic_vector(GOL_MAX_LEN - 1 downto 0);
		length_out : out std_logic_vector(3 downto 0)
	);
end entity gol_enc;

architecture beh of gol_enc is
	signal input_t1 : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal q_t2     : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal input_t2 : std_logic_vector(SMPL_LEN - 1 downto 0);
begin
	onesided : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				input_t1 <= (others => '0');
			else
				if input(7) = '1' then
					input_t1(7 downto 1) <= not input(6 downto 0);
					input_t1(0)          <= '1';
				else
					input_t1(7 downto 1) <= input(6 downto 0);
					input_t1(0)          <= '0';
				end if;
			end if;
		end if;
	end process onesided;

	shiftreg : entity work.gol_enc_shift port map(
			clk => clk,
			rst => rst,
			I   => input_t1,
			O   => q_t2
		);

	delay_input : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				input_t2 <= (others => '0');
			else
				input_t2 <= input_t1;
			end if;
		end if;
	end process delay_input;

	--	n_mux_modules : for n in GOL_MAX_LEN - 1 downto 0 generate
	--		mux_module : entity work.gol_enc_mux port map(
	--				D(0) => k_less(n),
	--				D(1) => k(n),
	--				Y    => reg_t1(n),
	--				S    => inReg(n)
	--			);
	--	end generate;


	reduce : process(clk) is
	begin
		if rising_edge(clk) then
			if rst = '1' then
				length_out <= (others => '0');
				output     <= (others => '0');
			else
				if (q_t2 > 6) then
					length_out <= "1111";
					output     <= "11111110" & input_t2;
				else
					length_out             <= q_t2(3 downto 0) + K_INT;
					output                 <= (others => '1');
					if K_INT = 0 then
						output(0)  <=  '0';
					else
						output(K_INT downto 0) <= '0' & input_t2(K_INT - 1 downto 0);
					end if;
				end if;
			end if;
		end if;
	end process reduce;
end beh;

