library ieee;

use ieee.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use work.defs.all;

entity loco is
	port(
		clk   : in  std_logic;
		rst   : in  std_logic;
		data  : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		P     : in  std_logic_vector(SMPL_LEN - 1 downto 0);
		Q_out : out std_logic_vector(SMPL_LEN - 1 downto 0)
	);
end entity loco;

architecture RTL of loco is
	signal e       : std_logic_vector(C_LEN downto 0);
	signal P_t0    : std_logic_vector(SMPL_LEN - 1 downto 0);
	signal data_t0 : std_logic_vector(SMPL_LEN - 1 downto 0);

begin
	adj_e : process(clk, rst) is
		variable temp_e1 : std_logic_vector(C_LEN downto 0);

	begin
		if rising_edge(clk) then
			if rst = '1' then
				e       <= (others => '0');
				P_t0    <= (others => '0');
				data_t0 <= (others => '0');
			else
				P_t0    <= P;
				data_t0 <= data;

				temp_e1 := (data_t0(7) & data_t0(7) & data_t0) - (P_t0(7) & P_t0(7) & P_t0);
				e       <= temp_e1(7) & temp_e1(7) & temp_e1(7 downto 0);
			end if;
		end if;
	end process adj_e;

	update_output : process(clk, rst) is
	begin
		if rst = '1' then
			Q_out <= (others => '0');
		elsif rising_edge(clk) then
			Q_out <= e(SMPL_LEN - 1 downto 0);
		end if;
	end process update_output;
end architecture RTL;
