library ieee;
use ieee.std_logic_1164.all;
use ieee.STD_LOGIC_ARITH.ALL;
use ieee.std_logic_unsigned.all;

entity wbuf is
	generic(constant IN_WIDTH   : integer := 16;
		      constant OUT_WIDTH  : integer := 32;
		      constant WBUF_WIDTH : integer := 64
	);
	port(
		clk         : in  std_ulogic;
		rst         : in  std_ulogic;

		-- INPUTS
		in_wbuf     : in  std_logic_vector(IN_WIDTH - 1 downto 0);
		word_length : in  std_logic_vector(4 downto 0);
		wbuf_enable : in  std_logic;

		-- OUTPUTS
		out_wbuf    : out std_logic_vector(OUT_WIDTH - 1 downto 0);
		wbuf_empty  : out std_logic;
		wbuf_write  : out std_logic
	);

end entity wbuf;

architecture RTL of wbuf is
begin
	G_ENC : process(clk, rst, wbuf_enable) is
		variable wbuffer      : std_logic_vector(WBUF_WIDTH - 1 downto 0);
		variable init_done    : boolean;
		variable num_elements : std_logic_vector(5 downto 0);
		variable write_pos    : std_logic_vector(5 downto 0);
		variable read_pos     : std_logic_vector(5 downto 0);
		variable shift_input  : std_logic_vector(in_wbuf'range);
	begin
		if (rst = '1') then
			wbuffer      := (others => '0');
			init_done    := false;
			num_elements := (others => '0');
			write_pos    := (others => '1');
			read_pos     := (others => '1');

			out_wbuf   <= (others => '0');
			wbuf_empty <= '0';
			wbuf_write <= '0';

		elsif (rising_edge(clk) and wbuf_enable = '1') then
			if (init_done) then
				shift_input(IN_WIDTH - 1 downto IN_WIDTH - conv_integer((word_length)) - 1) := in_wbuf(conv_integer((word_length)) downto 0);
				if (conv_integer(write_pos) - IN_WIDTH < 0) then
					wbuffer(conv_integer(write_pos) downto 0)                                                    := shift_input(IN_WIDTH - 1 downto IN_WIDTH - 1 - conv_integer(write_pos));
					wbuffer(WBUF_WIDTH - 1 downto WBUF_WIDTH - 1 - (IN_WIDTH - 1 - conv_integer(write_pos) - 1)) := shift_input(IN_WIDTH - 1 - conv_integer(write_pos) - 1 downto 0);
				else
					wbuffer(conv_integer(write_pos) downto conv_integer(write_pos) - IN_WIDTH + 1) := shift_input;
				end if;

				num_elements := num_elements + (word_length) + "1";
				write_pos    := write_pos - (word_length) - "1";

				-- to mem
				if (num_elements >= OUT_WIDTH) then
					out_wbuf     <= wbuffer(conv_integer(read_pos) downto conv_integer(read_pos) - OUT_WIDTH + 1);
					wbuf_write   <= '1';
					read_pos     := read_pos - "100000";
					num_elements := num_elements - "100000";
				else
					wbuf_write <= '0';
					out_wbuf   <= (others => '0');
				end if;
				wbuf_empty <= '0';
			else
				num_elements := (others => '0');
				wbuf_empty   <= '1';
				write_pos    := (others => '1');
				read_pos     := (others => '1');
				init_done    := true;
			end if;
		elsif (rising_edge(clk) and wbuf_enable = '0') then
			if (num_elements > 0) then
				out_wbuf     <= (wbuffer(conv_integer(read_pos) downto conv_integer(read_pos) - OUT_WIDTH + 1));
				wbuf_write   <= '1';
				wbuf_empty   <= '0';
				num_elements := (others => '0');
			else
				wbuf_empty <= '1';
				wbuf_write <= '0';
				out_wbuf   <= (others => '0');
				read_pos   := (others => '1');
				write_pos  := (others => '1');
				init_done  := false;
			end if;
		end if;
	end process;

end architecture RTL;
