#ifndef FELICS_H
#define FELICS_H

#include "def.h"
#include "imgArray.h"

const int NUM_MK = 9;

class Felics{
public:
	Felics(ImgArray* arr);
	void run();
	void computeLH();
	void computeRange();
	void encode();
	void contextModeling();
private:

	int i,j,k;
	int L,H,P,A,B,D,X;
	ImgArray *I;
	//ImgArray *O;
	bool inRange;
	bool below;
	int codeLength;
	int maxD;
	int cumulativeTotal[ALPHABET_SIZE*2][SS3][NUM_MK];
	int contextMk[ALPHABET_SIZE*2][SS3];
};

Felics::Felics(ImgArray* arr) 	: I(arr)
																, codeLength(0)
																, maxD(0)
{
	for(int ii = 0; ii < ALPHABET_SIZE*2; ii++){
		for(int kk = 0; kk < SS3; kk++){
			for(int jj = 0; jj < NUM_MK; jj++){
				cumulativeTotal[ii][kk][jj] = 0;
			}
			contextMk[ii][kk] = FIXED_GOLOMB;
		}

	}

}

void Felics::contextModeling(){

	for(int mk = 0; mk < NUM_MK; mk++){
		if(!inRange && below){
			cumulativeTotal[D][k][mk] += mk + 1 + (int) floor((double) (L-P-1) / pow((double) 2, mk));
		}else if (!inRange){
			cumulativeTotal[D][k][mk] += mk + 1 + (int) floor((double) (P-H-1) / pow((double) 2, mk));
		}
	}
//	contextMk[D][k] = minV(cumulativeTotal[D][k],NUM_MK);


	if(i == 1079 && j == 1919 && k == 2){
		for(int dd = 0; dd < 256; dd++){
			for(int kk = 0; kk < 3; kk++){
				for(int mk = 0; mk < NUM_MK; mk++){
					if(cumulativeTotal[dd][kk][mk] > maxD){
						maxD = cumulativeTotal[dd][kk][mk];
					}
				}
			}
		}
	}
}

void Felics::encode(){
	int sel = 117;
	int sel2 = 0;
	if(inRange){
		int N = D+1;
		int kk = (int)ceil(log2(N));


		int offset = (int)pow((double)2,kk) - N;
		int adjP;
		if((P-L) <= floor(D/2))
			adjP = D - (P-L)*2;
		else
			adjP = 2*(P-L) - 1 - D;

		codeLength++;
		if(adjP < offset){
			codeLength += kk-1;
		}else{
			codeLength += kk;
		}
	}else if(below){
		codeLength += 2;
		codeLength += contextMk[D][k] + 1 + (int) floor((double) (L-P-1) / pow((double) 2, contextMk[D][k]));
	}else{
		codeLength += 2;
		codeLength += contextMk[D][k] + 1 + (int) floor((double) (P-H-1) / pow((double) 2, contextMk[D][k]));
		if(i == sel2 && j == sel){
		}
	}
}


void Felics::run(){
	for(i = 0; i < SS1; i++){
		for(j = 0; j < SS2; j++){
			for(k = 0; k < SS3; k++){
				if(i == 0 && (j == 0 || j == 1)){
					codeLength += 8;
				}else{
					P = I->read(i,j).RGB[k];
					A = I->read(i,j).a->RGB[k];
					B = I->read(i,j).b->RGB[k];

					computeLH();
					computeRange();
					encode();
					contextModeling();
				}
			}
		}
		for(int ii = 0; ii < ALPHABET_SIZE; ii++){
			for(int kk = 0; kk < SS3; kk++){
				for(int jj = 0; jj < NUM_MK; jj++){
					//cumulativeTotal[ii][kk][jj] = 0;
				}
			}
		}
	}
	cout << "FELICS"<< endl;
	cout << "avg length: \t" << (double) codeLength / (SS1*SS2*SS3) << endl;
	cout << "tot length: \t" << codeLength << endl;
	cout << "max sum: \t" << maxD << endl;
}

void Felics::computeLH(){
	L = min(A,B);
	H = max(A,B);
	D = H-L;
}

void Felics::computeRange(){
	if(P < L){
		inRange = false;
		below = true;
	}else if(P > H){
		inRange = false;
		below = false;
	}else{
		inRange = true;
		below = false;
	}
}

#endif //FELICS_H
