#ifndef LINE_H
#define LINE_H

#include "def.h"
#include "imgArray.h"
#include "entropy.h"
#include "adaptiveList.h"
#include "codec.h"

class Line {
public:
	Line();
	Line(ImgArray *, cMod *, Entropy *, AdaptiveList *, enum CODEC, int, char*, int);
	~Line();
	//Line nextLine();
	void write(int col, pixBuffer);
	pixBuffer read(int col);
	void predictError();
	void run();
	void writeFile();
	void index();
	int absValue(enum CODEC);
	void writeBuffer();
	void error();
	void golomb();
private:
	pix P;
	pixBuffer X;
	int j, k;
	int de;
	enum CODEC codec;
	enum CODEC codecPred;
	int lineNum;
	ImgArray *I;
	Entropy *tab;
	AdaptiveList *adL;
	cMod *cM;
	pixBuffer *line;
	pixBuffer *startLine;
	pixBuffer zeroP;
	char* fileName;
	FILE* file;

	void context();

};

Line::Line() {
}
;

Line::Line(ImgArray *arr, cMod *cMod, Entropy *t, AdaptiveList *adL,
		enum CODEC c, int l, char* outputName, int de) {
	line = new pixBuffer[SS2];
	pixBuffer *itLine = line;
	this->adL = adL;
	tab = t;
	codec = c;
	codecPred = c;
	lineNum = l;
	I = arr;
	cM = cMod;
	this->de = de;

}

Line::~Line() {
	delete line;
}

void Line::write(int col, pixBuffer p) {
	this->line[col] = p;
}

pixBuffer Line::read(int col) {
	return this->line[col];
}

void Line::predictError() {

	X.X1[k] = predict(&P, k, codecPred);

	if (X.sign[k] == 1)
		X.X2[k] = X.X1[k] + cM->C[k][X.index[k]];
	else
		X.X2[k] = X.X1[k] - cM->C[k][X.index[k]];
}

void Line::error() {
	if (de == 1) {
		X.E[k] = P.RGB[k] - X.X1[k];
		X.E_noCtxt_temp[k] = P.RGB[k] - X.X1[k];
		if (X.sign[k] == 0) {
			X.E[k] = -X.E[k];
			X.E_noCtxt_temp[k] = -X.E_noCtxt_temp[k];
		}
		X.E_u8[k] = mod(X.E[k], 256);
		X.E_noCtxt[k] = mod(X.E_noCtxt_temp[k], 256);
		X.E_noCtxt_temp[k] = mod(X.E_noCtxt_temp[k], 256);
//		X.E[k] = mod(X.E[k]+128,256) - 128;

	} else {
		X.E[k] = mod(P.RGB[k] + 128, 256) - 128;
		if (X.sign[k] == 0)
			X.E_u8[k] = mod((X.X2[k] - P.RGB[k]), 256);
		else
			X.E_u8[k] = mod((X.X2[k] + P.RGB[k]), 256);

		I->writeValue(lineNum, j, k, X.E_u8[k]);
	}
}

void Line::writeBuffer() {
	line[j] = X;
}

void Line::run() {
	for (j = 0; j < SS2; j++) {
		P = I->read(lineNum, j);
		for (k = 0; k < SS3; k++) {
			int absV[CODEC_NUM - 1] = { 0 };
			if (j == 0 && codec == ADAPTIVE) {
				for (int ii = 0; ii < CODEC_NUM - 1; ii++) {
					absV[ii] = absValue((enum CODEC) ii);
				}
				codecPred = (enum CODEC) minV(absV, CODEC_NUM - 1);
				adL->add(codecPred, k);
			}
			index();
			predictError();
			error();
			writeBuffer();

//			//DEBUG
//			if (lineNum == 0 && j == 200) {
//				cout << "\tc: " << (int) cM->C[k][X.index[k]];
//				cout << "\tr: " << (int) P.RGB[k];
//				cout << "\t x1: " << (int)X.X1[k];
//				cout << "\t x2: " << (int)X.X2[k];
//				cout << "\t e: " << (int)X.E[k];
//				cout << "\t eu8: " << (int) X.E_u8[k] << endl;
//			}

			golomb();
			context();

			if (de == 1) {
				tab[0].entAdd(codec, X.E_u8[k]);
				tab[1].entAdd(codec, X.E_noCtxt[k]);
			}
		}
	}
}

void Line::golomb() {
	unsigned char N;
	int mk = 0;
//	mk = FIXED_GOLOMB;
	while ((cM->N[k][X.index[k]] * (int) pow((double) 2, mk))
			< cM->A[k][X.index[k]] && mk < 9)
		++mk;

	//mapping
//	if(mk == 0 && (cM->B[k][X.index[k]] * 2) <= -cM->N[k][X.index[k]])
//		if(-X.E[k] -1 < 0)
//			N = 2*abs(-X.E[k] -1) -1;
//		else
//			N = 2*abs(-X.E[k] -1);
//	else
	if (X.E[k] < 0)
		N = 2 * abs(X.E[k]) - 1;
	else
		N = 2 * abs(X.E[k]);

	int size;

	size = mk + 1 + (int) floor((double) N / pow((double) 2, mk));
	if ((int) floor((double) N / pow((double) 2, mk)) > 6)
		size = 16;
	cM->golombCount += size;

	if (size < 1)
		cout << endl << "Error: Gol size < 1 at line: " << lineNum << ", pixel: " << j << endl;
}

void Line::context() {
	cM->B[k][X.index[k]] += X.E[k];
	cM->A[k][X.index[k]] += abs(X.E[k]);
	cM->N[k][X.index[k]]++;

	if (cM->N[k][X.index[k]] > N0) {
		cM->N[k][X.index[k]] = 15;
		cM->A[k][X.index[k]] = (short) ceil(cM->A[k][X.index[k]] / 2);
		cM->B[k][X.index[k]] = (short) ceil(cM->B[k][X.index[k]] / 2);
	}

	if (cM->B[k][X.index[k]] <= -(cM->N[k][X.index[k]])) {
		if (cM->C[k][X.index[k]] > -300){
			cM->C[k][X.index[k]]--;
		}

		cM->B[k][X.index[k]] += cM->N[k][X.index[k]];
		if (cM->B[k][X.index[k]] <= -(cM->N[k][X.index[k]])) {
			cM->B[k][X.index[k]] = -(cM->N[k][X.index[k]]) + 1;
		}
	} else if (cM->B[k][X.index[k]] > 0) {
		if(cM->C[k][X.index[k]] < 300){
			cM->C[k][X.index[k]]++;
		}
		cM->B[k][X.index[k]] -= cM->N[k][X.index[k]];
		if (cM->B[k][X.index[k]] > 0) {
			cM->B[k][X.index[k]] = 0;
		}
	}

	if (cM->maxA < cM->A[k][X.index[k]]) {
		cM->maxA = cM->A[k][X.index[k]];
	}
	if (cM->maxB < abs(cM->B[k][X.index[k]])) {
		cM->maxB = abs(cM->B[k][X.index[k]]);
	}
	if (cM->maxC < abs(cM->C[k][X.index[k]])) {
		cM->maxC = abs(cM->C[k][X.index[k]]);
	}
	if (cM->maxN < cM->N[k][X.index[k]]) {
		cM->maxN = cM->N[k][X.index[k]];
	}
}

void Line::index() {
	signed int g[3];
	signed int q[3];
	int l, m, n;
	g[0] = P.b->RGB[k] - P.i->RGB[k];
//	g[2] = 0;
	g[0] = P.b->RGB[k] - P.d->RGB[k];
	g[1] = P.b->RGB[k] - P.c->RGB[k];

	for (l = 0; l < 3; l++) {
		if (g[l] <= -21)
			q[l] = -3;
		else if (g[l] <= -9)
			q[l] = -3;
		else if (g[l] <= -3)
			q[l] = -2;
		else if (g[l] <= -1)
			q[l] = -1;
		else if (g[l] <= 0)
			q[l] = 0;
		else if (g[l] <= 2)
			q[l] = 1;
		else if (g[l] <= 8)
			q[l] = 2;
		else if (g[l] <= 20)
			q[l] = 3;
		else
			q[l] = 3;
	}
	X.sign[k] = 1;
	m = 0;
	if (q[m] == 0)
		m++;
	if (q[m] == 0)
		m++;
	if (q[m] < 0) {
		X.sign[k] = 0;
		for (n = 0; n < 3; n++) {
			q[n] = -q[n];
		}
	}
	X.index[k] = ((q[0]) * 9 * 9 + (q[1]) * 9 + (q[2]));
}

int Line::absValue(enum CODEC cc) {
	pix* lX = I->getLine(lineNum);
	int sum = 0;
	for (int jj = 0; jj < SS2; jj++) {
//		for(int cc = 0; cc < CODEC_NUM -1; cc++){
		sum += abs(lX->RGB[k] - predict((lX++), k, cc));
//		}
	}
	return sum;
}

#endif //LINE_H
